@@ -11,9 +11,9 @@
 import pyutilib.component.core
 pyutilib.component.core.PluginGlobals.push_env( 'coopr.opt' )
 
-import ampl
-import glpsol
-import pico
-import pyomo
+import coopr.plugins.converter.ampl
+import coopr.plugins.converter.glpsol
+import coopr.plugins.converter.pico
+import coopr.plugins.converter.pyomo
 
 pyutilib.component.core.PluginGlobals.pop_env()
