@@ -11,14 +11,14 @@
 
 __all__ = ['PyomoMIPConverter']
 
-from coopr.opt.base import *
-from pico import PicoMIPConverter
-
 from pyutilib.component.core import *
 from pyutilib.component.config import *
 from pyutilib.component.executables import *
 import pyutilib.services
 
+from coopr.opt.base import *
+from coopr.plugins.converter.pico import PicoMIPConverter
+
 
 class PyomoMIPConverter(ManagedSingletonPlugin):
 
@@ -53,8 +53,9 @@ class PyomoMIPConverter(ManagedSingletonPlugin):
         """
 
         import coopr.pyomo
+
         capabilities = kwds.pop("capabilities", None)
-        generate_symbolic_labels = kwds.pop('generateSymbolicLabels',False)
+        symbolic_solver_labels = kwds.pop('symbolic_solver_labels',False)
 
         if isinstance(args[2], basestring):
             instance = None
@@ -64,18 +65,32 @@ class PyomoMIPConverter(ManagedSingletonPlugin):
         if args[1] is ProblemFormat.cpxlp:
             problem_filename = pyutilib.services.TempfileManager.create_tempfile(suffix = '.pyomo.lp')
             if instance is not None:
-                (problem_filename, varmap) = instance.write(filename=problem_filename, format=ProblemFormat.cpxlp, solver_capability=capabilities, generate_symbolic_labels=generate_symbolic_labels)
+                (problem_filename, varmap) = instance.write(filename=problem_filename,
+                                                            format=ProblemFormat.cpxlp,
+                                                            solver_capability=capabilities,
+                                                            symbolic_solver_labels=symbolic_solver_labels)
                 return (problem_filename,),varmap # no map file is necessary
             else:
-                model = coopr.pyomo.scripting.pyomo2lp(['--save-model',problem_filename,args[2]])
-                return (model.filename,),None
+                ans = coopr.pyomo.scripting.pyomo2lp(['--save-model',problem_filename,args[2]])
+                if ans.errorcode:
+                    raise RuntimeError("pyomo2lp conversion returned nonzero error code (%s)" % ans.errorcode)
+                model = ans.retval
+                problem_filename = model.filename
+                varmap = model.symbol_map
+                return (problem_filename,),varmap
 
         elif args[1] in [ProblemFormat.mps, ProblemFormat.nl]:
             problem_filename = pyutilib.services.TempfileManager.create_tempfile(suffix = '.pyomo.nl')
             if not instance is None:
-                (problem_filename, varmap) = instance.write(filename=problem_filename, format=ProblemFormat.nl, solver_capability=capabilities)
+                (problem_filename, varmap) = instance.write(filename=problem_filename, 
+                                                            format=ProblemFormat.nl, 
+                                                            solver_capability=capabilities,
+                                                            symbolic_solver_labels=symbolic_solver_labels)
             else:
-                model = coopr.pyomo.scripting.pyomo2nl(['--save-model',problem_filename,args[2]])
+                ans = coopr.pyomo.scripting.pyomo2nl(['--save-model',problem_filename,args[2]])
+                if ans.errorcode:
+                    raise RuntimeError("pyomo2lp conversion returned nonzero error code (%s)" % ans.errorcode)
+                model = ans.retval
                 problem_filename = model.filename
                 varmap = model.symbol_map
             if args[1] is ProblemFormat.nl:
@@ -94,10 +109,13 @@ class PyomoMIPConverter(ManagedSingletonPlugin):
         elif args[1] is ProblemFormat.osil:
             problem_filename = pyutilib.services.TempfileManager.create_tempfile(suffix='pyomo.osil')
             if instance:
-                (problem_filename, varmap) = instance.write(filename=problem_filename, format=ProblemFormat.osil, solver_capability=capabilities)
+                (problem_filename, varmap) = instance.write(filename=problem_filename,
+                                                            format=ProblemFormat.osil,
+                                                            solver_capability=capabilities,
+                                                            symbolic_solver_labels=symbolic_solver_labels)
                 return (problem_filename,), None
             else:
                 msg = 'There is currently no script conversion available from\n'
                 msg += 'Pyomo to OSiL format.'
 
-                raise NotImplementedError, msg
+                raise NotImplementedError(msg)
