@@ -10,18 +10,26 @@
 
 
 import re
-from coopr.opt.base import *
-from coopr.opt.results import *
-from coopr.opt.solver import *
+import os
+import copy
+
 import pyutilib.services
 import pyutilib.common
 import pyutilib.common
 import pyutilib.component.core
 import pyutilib.component.config
 import pyutilib.misc
-import mockmip
-import os
-import copy
+
+from coopr.opt.base import *
+from coopr.opt.base.solvers import _extract_version
+from coopr.opt.results import *
+from coopr.opt.solver import *
+from coopr.plugins.solvers.mockmip import MockMIP
+
+try:
+    unicode
+except:
+    basestring = str
 
 
 class ASL(SystemCallSolver):
@@ -37,19 +45,25 @@ class ASL(SystemCallSolver):
         kwds["type"] = "asl"
         SystemCallSolver.__init__(self, **kwds)
         #
-        # Setup valid problem formats, and valid results for each problem format
+        # Setup valid problem formats, and valid results for each problem format.
+        # Also set the default problem and results formats.
         #
         self._valid_problem_formats=[ProblemFormat.nl]
         self._valid_result_formats = {}
         self._valid_result_formats[ProblemFormat.nl] = [ResultsFormat.sol]
-
+        self.set_problem_format(ProblemFormat.nl)
+        
         # Note: Undefined capabilities default to 'None'
         self._capabilities = pyutilib.misc.Options()
         self._capabilities.linear = True
         self._capabilities.integer = True
+        self._capabilities.quadratic_objective = True
+        self._capabilities.quadratic_constraint = True
         self._capabilities.sos1 = True
         self._capabilities.sos2 = True
-        self._capabilities.sosn = True
+
+    def _default_results_format(self, prob_format):
+        return ResultsFormat.sol
 
     def executable(self):
         #
@@ -61,7 +75,7 @@ class ASL(SystemCallSolver):
                 raise pyutilib.component.config.OptionError('ERROR')
             pyutilib.services.register_executable(self.options.solver)
         except pyutilib.component.config.OptionError:
-            raise ValueError, "No solver option specified for ASL solver interface"
+            raise ValueError("No solver option specified for ASL solver interface")
         executable = pyutilib.services.registered_executable(self.options.solver)
         if executable is None:
             pyutilib.component.core.PluginGlobals.env().log.warning("Could not locate the '%s' executable, which is required for solver %s" % (self.options.solver, self.name))
@@ -69,7 +83,21 @@ class ASL(SystemCallSolver):
             return None
         return executable.get_path()
 
+    def version(self):
+        """
+        Returns a tuple describing the solver executable version.
+        """
+        solver_exec = self.executable()
+        if solver_exec is None:
+            return _extract_version('')
+        results = pyutilib.subprocess.run( [solver_exec,"-v"], timelimit=1 )
+        return _extract_version(results[1])
+
     def create_command_line(self, executable, problem_files):
+ 
+        assert(self._problem_format == ProblemFormat.nl)
+        assert(self._results_format == ResultsFormat.sol)
+
         #
         # Define log file
         #
@@ -83,30 +111,40 @@ class ASL(SystemCallSolver):
             else:
                 fname = tmp[0]
         self.soln_file = fname+".sol"
+
         #
         # Define results file
         #
-        if self._results_format is None or self._results_format == ResultsFormat.sol:
-            self.results_file = self.soln_file
+        self.results_file = self.soln_file
+        
         #
         # Define command line
         #
         env=copy.copy(os.environ)
 
-        # JDS: I am not sure why we did this test... mostly because if
-        # it ever returned false, 'proc' (now cmd) was never
-        # initialized, resulting in an exception.
-        #if self._problem_format is None or self._problem_format == ProblemFormat.nl:
-        #   ...
+        #
+        # Merge the COOPR_AMPLFUNC (externals defined within
+        # Coopr/Pyomo) with any user-specified external function
+        # libraries
+        #
+        if 'COOPR_AMPLFUNC' in env:
+            if 'AMPLFUNC' in env:
+                env['AMPLFUNC'] += "\n" + env['COOPR_AMPLFUNC']
+            else:
+                env['AMPLFUNC'] = env['COOPR_AMPLFUNC']
 
         cmd = [executable, '-s', problem_files[0]]
         if self._timer:
             cmd.insert(0, self._timer)
         
         # GAH: I am going to re-add the code by Zev that passed options through
-        # to the command line. I'm not sure what solvers this method of passing options
-        # through the envstr variable works for, but it does not seem to work for cplex
-        # or gurobi
+        # to the command line. Setting the environment variable in this way does
+        # NOT work for solvers like cplex and gurobi because the are looking for 
+        # an environment variable called cplex_options / gurobi_options. However
+        # the options.solver name for these solvers is cplexamp / gurobi_ampl 
+        # (which creates a cplexamp_options and gurobi_ampl_options env variable).
+        # Because of this, I think the only reliable way to pass options for any 
+        # solver is by using the command line
         opt=[]
         for key in self.options:
             if key is 'solver':
@@ -127,9 +165,6 @@ class ASL(SystemCallSolver):
             
         return pyutilib.misc.Bunch(cmd=cmd, log_file=self.log_file, env=env)
 
-    def _default_results_format(self, prob_format):
-        return ResultsFormat.sol
-
     def Xprocess_soln_file(self,results):
         """
         Process the SOL file
@@ -140,7 +175,7 @@ class ASL(SystemCallSolver):
             return
 
 
-class MockASL(ASL,mockmip.MockMIP):
+class MockASL(ASL,MockMIP):
     """A Mock ASL solver used for testing
     """
 
@@ -151,18 +186,18 @@ class MockASL(ASL,mockmip.MockMIP):
             ASL.__init__(self,**kwds)
         except pyutilib.common.ApplicationError: #pragma:nocover
             pass                        #pragma:nocover
-        mockmip.MockMIP.__init__(self,"asl")
+        MockMIP.__init__(self,"asl")
 
     def available(self, exception_flag=True):
         return ASL.available(self,exception_flag)
 
     def create_command_line(self,executable,problem_files):
         command = ASL.create_command_line(self,executable,problem_files)
-        mockmip.MockMIP.create_command_line(self,executable,problem_files)
+        MockMIP.create_command_line(self,executable,problem_files)
         return command
 
     def executable(self):
-        return mockmip.MockMIP.executable(self)
+        return MockMIP.executable(self)
 
     def _execute_command(self,cmd):
-        return mockmip.MockMIP._execute_command(self,cmd)
+        return MockMIP._execute_command(self,cmd)
