@@ -13,40 +13,54 @@ __all__ = ['CBC', 'MockCBC']
 import os
 import re
 import string
-from coopr.opt.base import *
-from coopr.opt.results import *
-from coopr.opt.solver import *
+import logging
+
+from six import iteritems
+
 import pyutilib.services
 import pyutilib.misc
 import pyutilib.common
 import pyutilib.component.core
 import pyutilib.subprocess
-import mockmip
-import logging
 
+from coopr.opt.base import *
+from coopr.opt.base.solvers import _extract_version
+from coopr.opt.results import *
+from coopr.opt.solver import *
+from coopr.plugins.solvers.mockmip import MockMIP
+
+logger = logging.getLogger('coopr.pyomo')
+
+global cbc_compiled_with_asl
 cbc_compiled_with_asl=None
-
-def configure():
+_cbc_version = None
+def configure_cbc():
     global cbc_compiled_with_asl
-    if cbc_compiled_with_asl is None:
-        # manually look for the cbc executable to prevent the
-        # CBC.execute() from logging an error when CBC is missing
-        if pyutilib.services.registered_executable("cbc") is None:
-            cbc_compiled_with_asl = False
-        else:
-            cbc_exec = CBC().executable()
-            results = pyutilib.subprocess.run(
-                [cbc_exec,"dummy","-AMPL","-stop"] )
-            if cbc_compiled_with_asl is not None:
-                raise IOError
-            cbc_compiled_with_asl = not ('No match for AMPL' in results[1])
+    global _cbc_version
+    if cbc_compiled_with_asl is not None:
+        return
+    cbc_compiled_with_asl = False
+    # manually look for the cbc executable to prevent the
+    # CBC.execute() from logging an error when CBC is missing
+    if pyutilib.services.registered_executable("cbc") is None:
+        return
+    cbc_exec = pyutilib.services.registered_executable("cbc").get_path()
+    results = pyutilib.subprocess.run( [cbc_exec,"-stop"], timelimit=1 )
+    _cbc_version = _extract_version(results[1])
+    if _cbc_version < (2,7,0,0):
+        logger.warning("found CBC version ("+('.'.join(str(i) for i in _cbc_version))+") < 2.7; ASL support disabled.")
+        logger.warning("consider upgrading CBC")
+        return
+    results = pyutilib.subprocess.run( 
+        [cbc_exec,"dummy","-AMPL","-stop"], timelimit=1 )
+    cbc_compiled_with_asl = not ('No match for AMPL' in results[1])
 
 
 class CBC(OptSolver):
     """The CBC LP/MIP solver
     """
 
-    pyutilib.component.core.alias('cbc', doc='The CBC MIP solver')
+    pyutilib.component.core.alias('cbc', doc='The CBC LP/MIP solver')
 
     def __new__(cls, *args, **kwds):
         try:
@@ -58,11 +72,18 @@ class CBC(OptSolver):
             mode = 'lp'
         #
         if mode  == 'lp':
-            return SolverFactory('_cbc_shell', **kwds)
+            opt = SolverFactory('_cbc_shell', **kwds)
+            opt.set_problem_format(ProblemFormat.cpxlp)
+            return opt
         #
         if mode == 'nl':
             if cbc_compiled_with_asl:
-                opt = SolverFactory('_asl', **kwds)
+                # CBC doesn't not accept all asl style command line options
+                # (-s in particular, which is required for streaming output of 
+                # all asl solvers). Therefore we need to send it through the cbc_shell
+                # instead of ASL
+                opt = SolverFactory('_cbc_shell',**kwds)
+                opt.set_problem_format(ProblemFormat.nl)
             else:
                 logging.getLogger('coopr.plugins').error('CBC solver is not compiled with ASL interface.')
                 return
@@ -91,7 +112,6 @@ class CBCSHELL(SystemCallSolver):
         #
         # Set up valid problem formats and valid results for each problem format
         #
-#        self._valid_problem_formats=[ProblemFormat.nl, ProblemFormat.cpxlp, ProblemFormat.mps]
         self._valid_problem_formats=[ProblemFormat.cpxlp, ProblemFormat.mps]
         if cbc_compiled_with_asl:
             self._valid_problem_formats.append(ProblemFormat.nl)
@@ -100,35 +120,30 @@ class CBCSHELL(SystemCallSolver):
         if cbc_compiled_with_asl:
             self._valid_result_formats[ProblemFormat.nl] = [ResultsFormat.sol]
         self._valid_result_formats[ProblemFormat.mps] = [ResultsFormat.soln]
+        self.set_problem_format(ProblemFormat.cpxlp)
 
         # Note: Undefined capabilities default to 'None'
         self._capabilities = pyutilib.misc.Options()
         self._capabilities.linear = True
         self._capabilities.integer = True
+        # The quadratic capabilities may be true but there is
+        # some weirdness in the solution file that this 
+        # plugin does not handle correctly  (extra variables 
+        # added that are not in the symbol map?)
+        self._capabilities.quadratic_objective = False
+        self._capabilities.quadratic_constraint = False
+        self._capabilities.sos1 = False
+        self._capabilities.sos2 = False
 
-    def _presolve(self, *args, **kwds):
-
-        # establish real "default" problem and results formats. these may be
-        # over-ridden in the base class solve (via keywords), but we should
-        # have real values by the time we're presolving.
-        if self._problem_format is None:
-
-            if cbc_compiled_with_asl and self._valid_problem_formats[0] is ProblemFormat.nl:
-                self._problem_format = ProblemFormat.nl
-            else:
-                self._problem_format = ProblemFormat.cpxlp
-
-        # in CBC, the results format is defined by the problem format;
-        # you can't vary them independently.
-        if cbc_compiled_with_asl and self._problem_format is ProblemFormat.nl:
-            self._results_format = ResultsFormat.sol
-        else:
-            # this really means "CBC-specific" format - not drawing from the
-            # log file itself (although additional information is contained there).
-            self._results_format = ResultsFormat.soln
-
-        # let the base class handle any remaining keywords/actions.
-        SystemCallSolver._presolve(self, *args, **kwds)
+    def _default_results_format(self, prob_format):
+        if prob_format == ProblemFormat.nl:
+            return ResultsFormat.sol
+        return ResultsFormat.soln
+        
+    # Nothing needs to be done here
+    #def _presolve(self, *args, **kwds):
+    #    # let the base class handle any remaining keywords/actions.
+    #    SystemCallSolver._presolve(self, *args, **kwds)
 
     def executable(self):
         executable = pyutilib.services.registered_executable("cbc")
@@ -138,6 +153,14 @@ class CBCSHELL(SystemCallSolver):
             return None
         return executable.get_path()
 
+    def version(self):
+        """
+        Returns a tuple describing the solver executable version.
+        """
+        if _cbc_version is None:
+            return _extract_version('')
+        return _cbc_version
+
     def create_command_line(self, executable, problem_files):
         #
         # Define the log file
@@ -160,10 +183,6 @@ class CBCSHELL(SystemCallSolver):
                 problem_filename_prefix = '.'.join(tmp[:-1])
             else:
                 problem_filename_prefix = tmp[0]
-
-        #if not cbc_compiled_with_asl:
-            #self._results_format = ResultsFormat.soln
-            #self.results_reader = None
         if self._results_format is ResultsFormat.sol:
             self.soln_file = problem_filename_prefix+".sol"
         else:
@@ -177,7 +196,7 @@ class CBCSHELL(SystemCallSolver):
         self.results_file = self.soln_file
 
         def _check_and_escape_options(options):
-            for key, val in self.options.iteritems():
+            for key, val in iteritems(self.options):
                 tmp_k = str(key)
                 _bad = ' ' in tmp_k
 
@@ -202,21 +221,21 @@ class CBCSHELL(SystemCallSolver):
         cmd = [ executable ]
         if self._timer:
             cmd.append(self._timer)
-
         if self._problem_format == ProblemFormat.nl:
             cmd.append(problem_files[0])
             cmd.append('-AMPL')
 
-            opts = ["stat=1", "printingOptions=all"]
             if self._timelimit is not None and self._timelimit > 0.0:
-                opts.append("sec=%s" % (self._timelimit,))
+                cmd.extend(['-sec', str(self._timelimit)])
             if "debug" in self.options:
-                opts.append("log=5")
+                cmd.extend(["-log","5"])
             for key, val in _check_and_escape_options(self.options):
-                opts.append("%s=%s" % ( key, val ))
-
-            env["CBC_options"] = " ".join(opts)            
-
+                if key is 'solver':
+                    continue
+                cmd.extend(['-'+key, val])
+            os.environ['cbc_options']="printingOptions=all"
+            #cmd.extend(["-printingOptions=all",
+                        #"-stat"])
         else:
             if self._timelimit is not None and self._timelimit > 0.0:
                 cmd.extend(['-sec', str(self._timelimit)])
@@ -227,7 +246,7 @@ class CBCSHELL(SystemCallSolver):
             cmd.extend(["-printingOptions", "all",
                         "-import", problem_files[0],
                         "-import",
-                        "-stat", "1",
+                        "-stat=1",
                         "-solve", 
                         "-solu", self.soln_file])
 
@@ -238,6 +257,12 @@ class CBCSHELL(SystemCallSolver):
         Process logfile
         """
         results = SolverResults()
+
+        # The logfile output for cbc when using nl files
+        # provides no information worth parsing here
+        if self._problem_format is ProblemFormat.nl:
+            return results
+
         #
         # Initial values
         #
@@ -350,7 +375,6 @@ class CBCSHELL(SystemCallSolver):
         return results
 
     def process_soln_file(self, results):
-
         # the only suffixes that we extract from CBC are
         # constraint duals and variable reduced-costs. scan
         # through the solver suffix list and throw an
@@ -366,7 +390,7 @@ class CBCSHELL(SystemCallSolver):
                 extract_reduced_costs = True
                 flag=True
             if not flag:
-                raise RuntimeError,"***CBC solver plugin cannot extract solution suffix="+suffix
+                raise RuntimeError("***CBC solver plugin cannot extract solution suffix="+suffix)
 
         # if dealing with SOL format files, we've already read
         # this via the base class reader functionality.
@@ -384,6 +408,7 @@ class CBCSHELL(SystemCallSolver):
 
         processing_constraints = None # None means header, True means constraints, False means variables.
         header_processed = False
+        range_duals = {}
         try:
             INPUT = open(self.soln_file,"r")
         except IOError:
@@ -391,7 +416,6 @@ class CBCSHELL(SystemCallSolver):
         for line in INPUT:
             tokens = re.split('[ \t]+',line.strip())
             # these are the only header entries CBC will generate (identified via browsing CbcSolver.cpp)
-
             if tokens[0] == "Optimal":
                 solution.status = SolutionStatus.optimal
                 solution.gap = 0.0
@@ -414,7 +438,7 @@ class CBCSHELL(SystemCallSolver):
                 return
 
             elif tokens[0] in ("Optimal", "Infeasible", "Unbounded", "Stopped", "Integer", "Status"):
-                print "***WARNING: CBC plugin currently not processing solution status="+tokens[0]+" correctly. Full status line is: "+string.strip(line)
+                print("***WARNING: CBC plugin currently not processing solution status="+tokens[0]+" correctly. Full status line is: "+string.strip(line))
 
             if tokens[0] in ("Optimal", "Infeasible", "Unbounded", "Stopped", "Integer", "Status"):
                 header_processed = True
@@ -425,7 +449,7 @@ class CBCSHELL(SystemCallSolver):
                 elif processing_constraints is True:
                     processing_constraints = False
                 else:
-                    raise RuntimeError, "CBC plugin encountered unexpected line=("+line.strip()+") in solution file="+self.soln_file+"; constraint and variable sections already processed!"
+                    raise RuntimeError("CBC plugin encountered unexpected line=("+line.strip()+") in solution file="+self.soln_file+"; constraint and variable sections already processed!")
 
             if (processing_constraints is True) and (extract_duals is True):
                 if len(tokens) == 4:
@@ -433,12 +457,17 @@ class CBCSHELL(SystemCallSolver):
                 elif (len(tokens) == 5) and tokens[0] == "**":
                     tokens = tokens[1:]
                 else:
-                    raise RuntimeError, "Unexpected line format encountered in CBC solution file - line="+line
+                    raise RuntimeError("Unexpected line format encountered in CBC solution file - line="+line)
 
                 constraint = tokens[1]
                 constraint_ax = eval(tokens[2]) # CBC reports the constraint row times the solution vector - not the slack.
                 constraint_dual = eval(tokens[3])
-                solution.constraint[constraint].dual = constraint_dual
+                if constraint.startswith('c_'):
+                    solution.constraint[constraint].dual = constraint_dual
+                elif constraint.startswith('r_l_'):
+                    range_duals.setdefault(constraint[4:],[0,0])[0] = constraint_dual
+                elif constraint.startswith('r_u_'):
+                    range_duals.setdefault(constraint[4:],[0,0])[1] = constraint_dual
 
             elif processing_constraints is False:
                 if len(tokens) == 4:
@@ -446,7 +475,7 @@ class CBCSHELL(SystemCallSolver):
                 elif (len(tokens) == 5) and tokens[0] == "**":
                     tokens = tokens[1:]
                 else:
-                    raise RuntimeError, "Unexpected line format encountered in CBC solution file - line="+line
+                    raise RuntimeError("Unexpected line format encountered in CBC solution file - line="+line)
 
                 variable_name = tokens[1]
                 variable_value = eval(tokens[2])
@@ -459,13 +488,22 @@ class CBCSHELL(SystemCallSolver):
                 pass
 
             else:
-                raise RuntimeError, "CBC plugin encountered unexpected line=("+line.strip()+") in solution file="+self.soln_file+"; expecting header, but found data!"
+                raise RuntimeError("CBC plugin encountered unexpected line=("+line.strip()+") in solution file="+self.soln_file+"; expecting header, but found data!")
 
         if not type(INPUT) is list:
             INPUT.close()
 
+        # For the range constraints, supply only the dual with the largest
+        # magnitude (at least one should always be numericly zero)
+        scon = solution.Constraint
+        for key,(ld,ud) in iteritems(range_duals):
+            if abs(ld) > abs(ud):
+                scon['r_l_'+key].dual = ld
+            else:
+                scon['r_u_'+key].dual = ud
 
-class MockCBC(CBCSHELL,mockmip.MockMIP):
+
+class MockCBC(CBCSHELL,MockMIP):
     """A Mock CBC solver used for testing
     """
 
@@ -476,21 +514,21 @@ class MockCBC(CBCSHELL,mockmip.MockMIP):
             CBCSHELL.__init__(self,**kwds)
         except pyutilib.common.ApplicationError: #pragma:nocover
             pass                        #pragma:nocover
-        mockmip.MockMIP.__init__(self,"cbc")
+        MockMIP.__init__(self,"cbc")
 
     def available(self, exception_flag=True):
         return CBCSHELL.available(self,exception_flag)
 
     def create_command_line(self,executable,problem_files):
         command = CBCSHELL.create_command_line(self,executable,problem_files)
-        mockmip.MockMIP.create_command_line(self,executable,problem_files)
+        MockMIP.create_command_line(self,executable,problem_files)
         return command
 
     def executable(self):
-        return mockmip.MockMIP.executable(self)
+        return MockMIP.executable(self)
 
     def _execute_command(self,cmd):
-        return mockmip.MockMIP._execute_command(self,cmd)
+        return MockMIP._execute_command(self,cmd)
 
     def _convert_problem(self,args,pformat,valid_pformats):
         if pformat in [ProblemFormat.mps, ProblemFormat.cpxlp, ProblemFormat.nl]:
