@@ -11,26 +11,36 @@
 
 import os
 import re
-from coopr.opt.base import *
-from coopr.opt.results import *
-from coopr.opt.solver import *
-import mockmip
-import pyutilib.services
-import pyutilib.common
-import pyutilib.misc
-import pyutilib.component.core
-import string
 import re
 import xml.dom.minidom
 import time
 import logging
 
+import pyutilib.services
+import pyutilib.common
+import pyutilib.misc
+import pyutilib.component.core
+
+from coopr.opt.base import *
+from coopr.opt.base.solvers import _extract_version
+from coopr.opt.results import *
+from coopr.opt.solver import *
+from coopr.pyomo.base.blockutil import has_discrete_variables
+from coopr.plugins.solvers.mockmip import MockMIP
+
+from six import iteritems
+from six.moves import xrange
+
+try:
+    unicode
+except:
+    basestring = unicode = str
 
 class CPLEX(OptSolver):
     """The CPLEX LP/MIP solver
     """
 
-    pyutilib.component.core.alias('cplex', doc='The CPLEX MIP solver')
+    pyutilib.component.core.alias('cplex', doc='The CPLEX LP/MIP solver')
 
     def __new__(cls, *args, **kwds):
         try:
@@ -88,15 +98,20 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
         self._valid_result_formats={}
         self._valid_result_formats[ProblemFormat.cpxlp] = [ResultsFormat.soln]
         self._valid_result_formats[ProblemFormat.mps] = [ResultsFormat.soln]
+        self.set_problem_format(ProblemFormat.cpxlp)
 
         # Note: Undefined capabilities default to 'None'
         self._capabilities = pyutilib.misc.Options()
         self._capabilities.linear = True
-        self._capabilities.quadratic = True
+        self._capabilities.quadratic_objective = True
+        self._capabilities.quadratic_constraint = True
         self._capabilities.integer = True
         self._capabilities.sos1 = True
         self._capabilities.sos2 = True
 
+    def _default_results_format(self, prob_format):
+        return ResultsFormat.soln
+
     #
     # CPLEX has a simple, easy-to-use warm-start capability.
     #
@@ -121,11 +136,11 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
 
         mst_file = open(self.warm_start_file_name, "w")
 
-        print >>mst_file, "<?xml version=\"1.0\" ?>"
-        print >>mst_file, "<CPLEXSolution version=\"1.0\">"
-        print >>mst_file, "<header/>"
-        print >>mst_file, "<quality/>"
-        print >>mst_file, "<variables>"
+        mst_file.write("<?xml version=\"1.0\" ?>\n")
+        mst_file.write("<CPLEXSolution version=\"1.0\">\n")
+        mst_file.write("<header/>\n")
+        mst_file.write("<quality/>\n")
+        mst_file.write("<variables>\n")
 
         # for each variable in the symbol_map, add a child to the
         # variables element.  Both continuous and discrete are accepted
@@ -140,16 +155,20 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
                     if (var.stale is True) or (var.fixed is True):
                         continue
                     name = self._symbol_map.getSymbol(var, _error_labeler)
-                    print >>mst_file, "<variable index=\"%d\" name=\"%s\" value=\"%f\" />" % (output_index, name, var.value)
+                    mst_file.write("<variable index=\"%d\" name=\"%s\" value=\"%f\" />\n" % (output_index, name, var.value))
                     output_index = output_index + 1
 
-        print >>mst_file, "</variables>"
-        print >>mst_file, "</CPLEXSolution>"
+        mst_file.write("</variables>\n")
+        mst_file.write("</CPLEXSolution>\n")
         mst_file.close()
 
     # over-ride presolve to extract the warm-start keyword, if specified.
     def _presolve(self, *args, **kwds):
 
+        # create a context in the temporary file manager for 
+        # this plugin - is "pop"ed in the _postsolve method.
+        pyutilib.services.TempfileManager.push()
+
         # if the first argument is a string (representing a filename),
         # then we don't have an instance => the solver is being applied
         # to a file.
@@ -161,7 +180,7 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
         # mechanism, invoked warm_start() with a instance to create the warm start file.
         if (self.warm_start_solve is True) and (isinstance(args[0],basestring) is True):
             pass # we assume the user knows what they are doing...
-        elif (self.warm_start_solve is True) and (isinstance(args[0],basestring) is False) and (args[0].has_discrete_variables() is True):
+        elif (self.warm_start_solve is True) and (isinstance(args[0],basestring) is False) and (has_discrete_variables(args[0]) is True):
            # assign the name of the warm start file *before* calling the base class
            # presolve - the base class method ends up creating the command line,
            # and the warm start file-name is (obviously) needed there.
@@ -186,12 +205,12 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
                         "CPLEX _presolve method can only handle a single "
                         "problem instance - %s were supplied" % (len(args),))
 
-                if args[0].has_discrete_variables() is True:
+                if has_discrete_variables(args[0]) is True:
                     start_time = time.time()
                     self.warm_start(args[0])
                     end_time = time.time()
                     if self._report_timing is True:
-                        print "Warm start write time= %.2f seconds" % (end_time-start_time)
+                        print("Warm start write time= %.2f seconds" % (end_time-start_time))
 
     def executable(self):
         executable = pyutilib.services.registered_executable("cplex")
@@ -201,6 +220,16 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
             return None
         return executable.get_path()
 
+    def version(self):
+        """
+        Returns a tuple describing the solver executable version.
+        """
+        solver_exec = self.executable()
+        if solver_exec is None:
+            return _extract_version('')
+        results = pyutilib.subprocess.run( [solver_exec,'-c','quit'], timelimit=1 )
+        return _extract_version(results[1])
+
     def create_command_line(self,executable,problem_files):
 
         #
@@ -214,14 +243,7 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
         # As indicated above, contains (in XML) both the solution and solver status.
         #
         self.soln_file = pyutilib.services.TempfileManager.create_tempfile(suffix = '.cplex.sol')
-
-        #
-        # Define results file
-        #
-        if self._results_format is None or self._results_format == ResultsFormat.soln:
-            self.results_file = self.soln_file
-        elif self._results_format == ResultsFormat.sol:
-            self.results_file = self.sol_file
+        self.results_file = self.soln_file
 
         #
         # Write the CPLEX execution script
@@ -255,24 +277,19 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
 
         # dump the script and warm-start file names for the
         # user if we're keeping files around.
-        if self.keepFiles:
+        if self.keepfiles:
             script_fname = pyutilib.services.TempfileManager.create_tempfile(suffix = '.cplex.script')
             tmp = open(script_fname,'w')
             tmp.write(script)
             tmp.close()
             
-            print "Solver script file=" + script_fname
+            print("Solver script file=" + script_fname)
             if (self.warm_start_solve is True) and (self.warm_start_file_name is not None):
-                print "Solver warm-start file=" + self.warm_start_file_name
+                print("Solver warm-start file=" + self.warm_start_file_name)
 
         #
         # Define command line
         #
-        # JDS: I am not sure why we did this test... mostly because if
-        # it ever returned false, 'proc' was never initialized, resulting
-        # in an exception.
-        #if self._problem_format in [ProblemFormat.cpxlp, ProblemFormat.mps]:
-        #    proc = self._timer + " " + self.executable()
         cmd = [self.executable()]
         if self._timer:
             cmd.insert(0, self._timer)
@@ -413,6 +430,9 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
         extract_duals = False
         extract_slacks = False
         extract_reduced_costs = False
+        extract_rc = False
+        extract_lrc = False
+        extract_urc = False
         for suffix in self.suffixes:
             flag=False
             if re.match(suffix,"dual"):
@@ -423,14 +443,24 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
                 flag=True
             if re.match(suffix,"rc"):
                 extract_reduced_costs = True
+                extract_rc = True
+                flag=True
+            if re.match(suffix,"lrc"):
+                extract_reduced_costs = True
+                extract_lrc = True
+                flag=True
+            if re.match(suffix,"urc"):
+                extract_reduced_costs = True
+                extract_urc = True
                 flag=True
             if not flag:
-                raise RuntimeError,"***The CPLEX solver plugin cannot extract solution suffix="+suffix
+                raise RuntimeError("***The CPLEX solver plugin cannot extract solution suffix="+suffix)
 
-        lp_solution = False
         if not os.path.exists(self.soln_file):
             return
 
+        range_duals = {}
+        range_slacks = {}
         soln = Solution()
         soln.objective['__default_objective__'].value=None
         soln_variable = soln.variable # caching for efficiency
@@ -443,7 +473,7 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
             line = line.lstrip('<?/')
             line = line.rstrip('/>?')
             tokens=line.split(' ')
-
+            
             if tokens[0] == "variable":
                 variable_name = None
                 variable_value = None
@@ -466,35 +496,51 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
                     variable = soln_variable[variable_name] = {"Value" : float(variable_value), "Id" : len(soln_variable)}
                     if (variable_reduced_cost is not None) and (extract_reduced_costs is True):
                         try:
-                            variable["Rc"] = float(variable_reduced_cost)
+                            if extract_rc is True:
+                                variable["Rc"] = float(variable_reduced_cost)
                             if variable_status is not None:
-                                if variable_status == "LL":
-                                    variable["Lrc"] = float(variable_reduced_cost)
-                                else:
-                                    variable["Lrc"] = 0.0
-                                if variable_status == "UL":
-                                    variable["Urc"] = float(variable_reduced_cost)
-                                else:
-                                    variable["Urc"] = 0.0
+                                if extract_lrc is True:
+                                    if variable_status == "LL":
+                                        variable["Lrc"] = float(variable_reduced_cost)
+                                    else:
+                                        variable["Lrc"] = 0.0
+                                if extract_urc is True:
+                                    if variable_status == "UL":
+                                        variable["Urc"] = float(variable_reduced_cost)
+                                    else:
+                                        variable["Urc"] = 0.0
                         except:
-                            raise ValueError, "Unexpected reduced-cost value="+str(variable_reduced_cost)+" encountered for variable="+variable_name
+                            raise ValueError("Unexpected reduced-cost value="+str(variable_reduced_cost)+" encountered for variable="+variable_name)
             elif (tokens[0] == "constraint") and ((extract_duals is True) or (extract_slacks is True)):
-                constraint_name = None
-                constraint_dual = None
-                constaint = None # cache the solution constraint reference, as the getattr is expensive.
+                range = False
+                rlabel = None
+                rkey = None
                 for i in xrange(1,len(tokens)):
                     field_name =  tokens[i].split('=')[0]
                     field_value = tokens[i].split('=')[1].lstrip("\"").rstrip("\"")
                     if field_name == "name":
-                        constraint_name = field_value
-                        constraint = soln.constraint[constraint_name]
+                        if field_value.startswith('c_'):
+                            constraint = soln.constraint[field_value]
+                        elif field_value.startswith('r_l_'):
+                            range = True
+                            rlabel = field_value[4:]
+                            rkey = 0
+                        elif field_value.startswith('r_u_'):
+                            range = True
+                            rlabel = field_value[4:]
+                            rkey = 1
                     elif (extract_duals is True) and (field_name == "dual"): # for LPs
-                        constraint.dual = float(field_value)
+                        if range is False:
+                            constraint.dual = float(field_value)
+                        else:
+                            range_duals.setdefault(rlabel,[0,0])[rkey] = float(field_value)
                     elif (extract_slacks is True) and (field_name == "slack"): # for MIPs
-                        constraint.slack = float(field_value)
+                        if range is False:
+                            constraint.slack = float(field_value)
+                        else:
+                            range_slacks.setdefault(rlabel,[0,0])[rkey] = float(field_value)
             elif tokens[0].startswith("problemName"):
-                filename = (string.strip(tokens[0].split('=')[1])).lstrip("\"").rstrip("\"")
-                #print "HERE",filename
+                filename = (tokens[0].split('=')[1].strip()).lstrip("\"").rstrip("\"")
                 results.problem.name = os.path.basename(filename)
                 if '.' in results.problem.name:
                     results.problem.name = results.problem.name.split('.')[0]
@@ -518,10 +564,10 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
                 tINPUT.close()
 
             elif tokens[0].startswith("objectiveValue"):
-                objective_value = (string.strip(tokens[0].split('=')[1])).lstrip("\"").rstrip("\"")
+                objective_value = (tokens[0].split('=')[1].strip()).lstrip("\"").rstrip("\"")
                 soln.objective['__default_objective__'].value = float(objective_value)
             elif tokens[0].startswith("solutionStatusValue"):
-               pieces = string.split(tokens[0],"=")
+               pieces = tokens[0].split("=")
                solution_status = eval(pieces[1])
                # solution status = 1 => optimal
                # solution status = 3 => infeasible
@@ -531,19 +577,20 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
                   elif solution_status == 3:
                     soln.status = SolutionStatus.infeasible
                     soln.gap = None                  
-                  elif solution_status >= 4: 
+                  else:
                       # we are flagging anything with a solution status >= 4 as an error, to possibly
                       # be over-ridden as we learn more about the status (e.g., due to time limit exceeded).
                       soln.status = SolutionStatus.error
                       soln.gap = None
             elif tokens[0].startswith("solutionStatusString"):
-                solution_status = (string.strip(" ".join(tokens).split('=')[1])).lstrip("\"").rstrip("\"")
+                solution_status = ((" ".join(tokens).split('=')[1]).strip()).lstrip("\"").rstrip("\"")
                 if solution_status in ["optimal", "integer optimal solution", "integer optimal, tolerance"]:
                     soln.status = SolutionStatus.optimal
                     soln.gap = 0.0
                     results.problem.lower_bound = soln.objective['__default_objective__'].value
                     results.problem.upper_bound = soln.objective['__default_objective__'].value
-                    mip_problem=True
+                    if "integer" in solution_status:
+                        mip_problem=True
                 elif solution_status in ["infeasible"]:
                     soln.status = SolutionStatus.infeasible
                     soln.gap = None
@@ -553,11 +600,11 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
                     time_limit_exceeded = True
             elif tokens[0].startswith("MIPNodes"):
                 if mip_problem:
-                    n = eval(eval(string.strip(" ".join(tokens).split('=')[1])).lstrip("\"").rstrip("\""))
+                    n = eval(eval((" ".join(tokens).split('=')[1]).strip()).lstrip("\"").rstrip("\""))
                     results.solver.statistics.branch_and_bound.number_of_created_subproblems=n
                     results.solver.statistics.branch_and_bound.number_of_bounded_subproblems=n
             elif tokens[0].startswith("primalFeasible") and (time_limit_exceeded is True):
-                primal_feasible = int((string.strip(" ".join(tokens).split('=')[1])).lstrip("\"").rstrip("\""))
+                primal_feasible = int(((" ".join(tokens).split('=')[1]).strip()).lstrip("\"").rstrip("\""))
                 if primal_feasible == 1:
                     soln.status = SolutionStatus.feasible
                     if (results.problem.sense == ProblemSense.minimize):
@@ -576,6 +623,21 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
         if self._gap is not None:
             soln.gap = self._gap
 
+        # For the range constraints, supply only the dual with the largest
+        # magnitude (at least one should always be numericly zero)
+        scon = soln.Constraint
+        for key,(ld,ud) in iteritems(range_duals):
+            if abs(ld) > abs(ud):
+                scon['r_l_'+key].dual = ld
+            else:
+                scon['r_u_'+key].dual = ud
+        # slacks
+        for key,(ls,us) in iteritems(range_slacks):
+            if abs(ls) > abs(us):
+                scon['r_l_'+key].slack = ls
+            else:
+                scon['r_u_'+key].slack = us
+
         if not results.solver.status is SolverStatus.error:
             if results.solver.termination_condition in [TerminationCondition.unknown,
                                                         #TerminationCondition.maxIterations,
@@ -615,10 +677,17 @@ class CPLEXSHELL(ILMLicensedSystemCallSolver):
                 pass
 
         # let the base class deal with returning results.
-        return ILMLicensedSystemCallSolver._postsolve(self)
+        results = ILMLicensedSystemCallSolver._postsolve(self)
 
+        # finally, clean any temporary files registered with the temp file
+        # manager, created populated *directly* by this plugin. does not
+        # include, for example, the execution script. but does include 
+        # the warm-start file. 
+        pyutilib.services.TempfileManager.pop(remove=not self.keepfiles)
 
-class MockCPLEX(CPLEXSHELL,mockmip.MockMIP):
+        return results
+
+class MockCPLEX(CPLEXSHELL,MockMIP):
     """A Mock CPLEX solver used for testing
     """
 
@@ -629,21 +698,21 @@ class MockCPLEX(CPLEXSHELL,mockmip.MockMIP):
             CPLEXSHELL.__init__(self, **kwds)
         except pyutilib.common.ApplicationError: #pragma:nocover
             pass                        #pragma:nocover
-        mockmip.MockMIP.__init__(self,"cplex")
+        MockMIP.__init__(self,"cplex")
 
     def available(self, exception_flag=True):
         return CPLEXSHELL.available(self,exception_flag)
 
     def create_command_line(self,executable,problem_files):
         command = CPLEXSHELL.create_command_line(self,executable,problem_files)
-        mockmip.MockMIP.create_command_line(self,executable,problem_files)
+        MockMIP.create_command_line(self,executable,problem_files)
         return command
 
     def executable(self):
-        return mockmip.MockMIP.executable(self)
+        return MockMIP.executable(self)
 
     def _execute_command(self,cmd):
-        return mockmip.MockMIP._execute_command(self,cmd)
+        return MockMIP._execute_command(self,cmd)
 
 
 pyutilib.services.register_executable(name="cplex")
