@@ -10,33 +10,46 @@
 
 
 import os
+import sys
 import re
-
-from coopr.opt.base import *
-from coopr.opt.results import *
-from coopr.opt.solver import *
-
-from coopr.pyomo.base.numvalue import value
-
-import mockmip
-import pyutilib.services
-import pyutilib.common
-import pyutilib.misc
-import pyutilib.component.core
 import string
-
 import xml.dom.minidom
-
 import time
 
+from six import itervalues, iterkeys, iteritems, advance_iterator
+from six.moves import xrange
+
+_cplex_version = None
 try:
     import cplex
     from cplex.exceptions import CplexError
+    # create a version tuple of length 4
+    _cplex_version = tuple(int(i) for i in cplex.Cplex().get_version().split('.'))
+    while(len(_cplex_version) < 4):
+        _cplex_version += (0,)
+    _cplex_version = _cplex_version[:4]
     cplex_import_available=True
 except ImportError:
     cplex_import_available=False
-    
-    
+
+import pyutilib.services
+import pyutilib.common
+import pyutilib.component.core
+from pyutilib.misc import Bunch, Options
+
+from coopr.opt.base import *
+from coopr.opt.base.solvers import _extract_version
+from coopr.opt.results import *
+from coopr.opt.solver import *
+from coopr.pyomo.base import SymbolMap, NumericLabeler, Suffix
+from coopr.pyomo.base.numvalue import value
+from coopr.pyomo.base.block import active_subcomponents_generator, active_subcomponents_data_generator
+
+try:
+    unicode
+except:
+    basestring = str
+
 class ModelSOS(object):
     def __init__(self):
         self.sosType = {}
@@ -98,9 +111,10 @@ class CPLEXDirect(OptSolver):
         self._active_cplex_instance = None
 
         # Note: Undefined capabilities default to 'None'
-        self._capabilities = pyutilib.misc.Options()
+        self._capabilities = Options()
         self._capabilities.linear = True
-        self._capabilities.quadratic = True
+        self._capabilities.quadratic_objective = True
+        self._capabilities.quadratic_constraint = True
         self._capabilities.integer = True
         self._capabilities.sos1 = True
         self._capabilities.sos2 = True
@@ -112,17 +126,14 @@ class CPLEXDirect(OptSolver):
             return cplex_import_available
         else:
             if cplex_import_available is False:
-                raise ApplicationError, "No CPLEX <-> Python bindings available - CPLEX direct solver functionality is not available"
+                raise ApplicationError("No CPLEX <-> Python bindings available - CPLEX direct solver functionality is not available")
             else:
                 return True
 
-    class _numeric_labeler(object):
-        def __init__(self):
-            self.id = 0
-        def __call__(self, obj):
-            self.id += 1
-            return str(self.id)
-
+    def version(self):
+        if _cplex_version is None:
+            return _extract_version('')
+        return _cplex_version
 
     #
     # ultimately, this utility should go elsewhere - perhaps on the PyomoModel itself.
@@ -133,11 +144,10 @@ class CPLEXDirect(OptSolver):
         from coopr.pyomo.base import Var
         from coopr.pyomo.base.set_types import IntegerSet, BooleanSet
 
-        for variable in instance.active_components(Var).values():
-
-            if (isinstance(variable.domain, IntegerSet) is True) or (isinstance(variable.domain, BooleanSet) is True):
-
-                return True
+        for block in instance.all_blocks():
+            for variable in active_subcomponents_generator(block,Var):
+                if (isinstance(variable.domain, IntegerSet) is True) or (isinstance(variable.domain, BooleanSet) is True):
+                    return True
 
         return False
 
@@ -153,7 +163,7 @@ class CPLEXDirect(OptSolver):
         elif exp.is_constant() or exp.is_fixed():
             return exp()
         else:
-            raise ValueError, "ERROR: nonconstant bound: " + str(exp)
+            raise ValueError("ERROR: nonconstant bound: " + str(exp))
 
     #
     # CPLEX requires objective expressions to be specified via something other than a sparse pair!
@@ -168,7 +178,7 @@ class CPLEXDirect(OptSolver):
 
         hash_to_variable_map = expression[-1]
 
-        for var_hash, var_coefficient in expression[1].iteritems():
+        for var_hash, var_coefficient in iteritems(expression[1]):
 
             variable_name = self._symbol_map.getSymbol(hash_to_variable_map[var_hash], labeler)
 
@@ -188,6 +198,41 @@ class CPLEXDirect(OptSolver):
             expr = cplex.SparsePair(ind=variables, val=coefficients)
             return expr, offset
 
+    #
+    # CPLEX requires objective expressions to be specified via something other than a sparse pair!
+    # NOTE: The returned offset is guaranteed to be a float.
+    # NOTE: This function is a variant of the above, specialized for LinearCanonicalRepn objects.
+    #
+    def _encode_constraint_body_linear_specialized(self, linear_repn, labeler, as_pairs=False):
+
+        variables = [] # string names of variables
+        coefficients = [] # variable coefficients
+
+        pairs = []
+
+        for i in xrange(0,len(linear_repn.linear)):
+
+            var_coefficient = linear_repn.linear[i]
+            var_value = linear_repn.variables[i]
+
+            variable_name = self._symbol_map.getSymbol(var_value, labeler)
+
+            if as_pairs is True:
+                pairs.append((variable_name, var_coefficient))
+            else:
+                variables.append(variable_name)
+                coefficients.append(var_coefficient)
+
+        offset=0.0
+        if linear_repn.constant != None:
+            offset = linear_repn.constant
+
+        if as_pairs is True:
+            return pairs, offset
+        else:
+            expr = cplex.SparsePair(ind=variables, val=coefficients)
+            return expr, offset
+
     #
     #Handle quadratic constraints and objectives
     #
@@ -201,14 +246,15 @@ class CPLEXDirect(OptSolver):
 
         hash_to_variable_map = expression[-1]
 
-        for vars, coeff in expression[2].iteritems():
+        for vrs, coeff in iteritems(expression[2]):
 
-            if len(vars)==2:
-                var1 = self._symbol_map.getSymbol(hash_to_variable_map[vars.keys()[0]], labeler)
-                var2 = self._symbol_map.getSymbol(hash_to_variable_map[vars.keys()[1]], labeler)
+            if len(vrs)==2:
+                variable_hash_iter = iterkeys(vrs)
+                var1 = self._symbol_map.getSymbol(hash_to_variable_map[advance_iterator(variable_hash_iter)], labeler)
+                var2 = self._symbol_map.getSymbol(hash_to_variable_map[advance_iterator(variable_hash_iter)], labeler)
             else:
-                variable_hash_iter = vars.iterkeys()
-                var1 = var2  = self._symbol_map.getSymbol(hash_to_variable_map[variable_hash_iter.next()], labeler)
+                variable_hash_iter = iterkeys(vrs)
+                var1 = var2  = self._symbol_map.getSymbol(hash_to_variable_map[advance_iterator(variable_hash_iter)], labeler)
 
             if as_triples is True:
                 triples.append((var1, var2, is_obj*coeff))
@@ -229,22 +275,23 @@ class CPLEXDirect(OptSolver):
     #
     def _populate_cplex_instance(self, pyomo_instance):
 
-        labeler = CPLEXDirect._numeric_labeler()
+        labeler = NumericLabeler('x')
         self._symbol_map = SymbolMap(pyomo_instance)
 
-        from coopr.pyomo.base import Var, Objective, Constraint, IntegerSet, BooleanSet, SOSConstraint
+        from coopr.pyomo.base import Var, Objective, Constraint, ConstraintList, IntegerSet, BooleanSet, SOSConstraint
         from coopr.pyomo.base.objective import minimize, maximize
         from coopr.pyomo.expr import canonical_is_constant
-
-        # TBD
-        quadratic = False
+        from coopr.pyomo import LinearCanonicalRepn
+    
+        quadratic_constraints = False
+        quadratic_objective = False
         used_sos_constraints = False
         cplex_instance = None
         try:
             cplex_instance = cplex.Cplex()
-        except CplexError, exc:
-            print exc
-            raise ValueError, "TBD - FAILED TO CREATE CPLEX INSTANCE!"
+        except CplexError:
+            print(sys.exc_info()[1])
+            raise ValueError("TBD - FAILED TO CREATE CPLEX INSTANCE!")
 
         # cplex wants the caller to set the problem type, which is (for current
         # purposes) strictly based on variable type counts.
@@ -258,31 +305,28 @@ class CPLEXDirect(OptSolver):
         var_ubs = []
         var_types = []
 
-        all_blocks = pyomo_instance.all_blocks()
-        for block in all_blocks:
-            for variable in block.active_components(Var).itervalues():
-                for ndx in variable:
-                    var = variable[ndx]
-                    if (not var.active) or var.fixed:
-                        continue
-                    var_names.append(self._symbol_map.getSymbol( var, labeler ))
-                    if var.lb is None:
-                        var_lbs.append(-cplex.infinity)
-                    else:
-                        var_lbs.append(value(var.lb))
-                    if var.ub is None:
-                        var_ubs.append(cplex.infinity)
-                    else:
-                        var_ubs.append(value(var.ub))
-                    if isinstance(variable.domain, IntegerSet):
-                        var_types.append(cplex_instance.variables.type.integer)
-                        num_integer_variables += 1
-                    elif isinstance(variable.domain, BooleanSet):
-                        var_types.append(cplex_instance.variables.type.binary)
-                        num_binary_variables += 1
-                    else:
-                        var_types.append(cplex_instance.variables.type.continuous)
-                        num_continuous_variables += 1
+        for block in pyomo_instance.all_blocks():
+            for var in active_subcomponents_data_generator(block,Var):
+                if (not var.active) or var.fixed:
+                    continue
+                var_names.append(self._symbol_map.getSymbol( var, labeler ))
+                if var.lb is None:
+                    var_lbs.append(-cplex.infinity)
+                else:
+                    var_lbs.append(value(var.lb))
+                if var.ub is None:
+                    var_ubs.append(cplex.infinity)
+                else:
+                    var_ubs.append(value(var.ub))
+                if isinstance(var.domain, IntegerSet):
+                    var_types.append(cplex_instance.variables.type.integer)
+                    num_integer_variables += 1
+                elif isinstance(var.domain, BooleanSet):
+                    var_types.append(cplex_instance.variables.type.binary)
+                    num_binary_variables += 1
+                else:
+                    var_types.append(cplex_instance.variables.type.continuous)
+                    num_continuous_variables += 1
 
         cplex_instance.variables.add(names=var_names, lb=var_lbs, ub=var_ubs, types=var_types)
         
@@ -299,39 +343,57 @@ class CPLEXDirect(OptSolver):
         qrhss = []
         qnames = []
 
-        for block in all_blocks:
-            for constraint in block.active_components(Constraint).itervalues():
+        for block in pyomo_instance.all_blocks():
+
+            block_repn_suffix = block.subcomponent("repn")
+            if (block_repn_suffix is None) or (not block_repn_suffix.type() is Suffix) or (not block_repn_suffix.active is True):
+                block_repn_suffix = None
+
+            for constraint in active_subcomponents_generator(block, Constraint):
                 if constraint.trivial:
                     continue
 
-                for cndx in constraint: # TBD: more efficient looping here.
-                    con = constraint[cndx]
+                for con in itervalues(constraint): # TBD: more efficient looping here.
                     if not con.active:
                         continue
 
+                    con_repn = None
+                    if block_repn_suffix is not None:
+                        con_repn = block_repn_suffix.getValue(con)
+
                     # There are conditions, e.g., when fixing variables, under which
                     # a constraint block might be empty.  Ignore these, for both
                     # practical reasons and the fact that the CPLEX LP format
                     # requires a variable in the constraint body.  It is also
                     # possible that the body of the constraint consists of only a
                     # constant, in which case the "variable" of
-                    if canonical_is_constant(con.repn):
-                        continue
+                    if isinstance(con_repn, LinearCanonicalRepn):
+                        if con_repn.linear == None:
+                           continue
+                    else:
+                       if canonical_is_constant(con_repn):
+                           continue
 
                     name=self._symbol_map.getSymbol(con,labeler)
                     expr=None
                     qexpr=None
+
                     #Linear constraints
-                    if 1 in con.repn:
-                        expr, offset = self._encode_constraint_body_linear(con.repn,labeler)
+                    quadratic = False
+                    if isinstance(con_repn, LinearCanonicalRepn):
+                        expr, offset = self._encode_constraint_body_linear_specialized(con_repn, labeler)
+                    elif 2 in con_repn:
+                        quadratic=True
+                    elif 1 in con_repn:
+                        expr, offset = self._encode_constraint_body_linear(con_repn, labeler)
 
                     #Quadratic constraints
-                    if 2 in con.repn:
+                    if quadratic is True:
                         if expr is None:
                             expr = cplex.SparsePair(ind=[0],val=[0.0])
-                        quadratic = True
+                        quadratic_constraints = True
 
-                        qexpr = self._encode_constraint_body_quadratic(con.repn,labeler)
+                        qexpr = self._encode_constraint_body_quadratic(con_repn,labeler)
                         qnames.append(name)
 
                         if con._equality:
@@ -342,6 +404,7 @@ class CPLEXDirect(OptSolver):
                             qrhss.append(bound)
 
                         elif con.lower is not None:
+                            assert con.upper is not None
                             qsenses.append('G')
                             bound_expr = con.lower
                             bound = self._evaluate_bound(bound_expr)
@@ -360,7 +423,7 @@ class CPLEXDirect(OptSolver):
                         names.append(name)
                         expressions.append(expr)
 
-                        if constraint[cndx]._equality:
+                        if con._equality:
                             # equality constraint.
                             senses.append('E')
                             bound_expr = con.lower
@@ -400,17 +463,11 @@ class CPLEXDirect(OptSolver):
         sos1 = self._capabilities.sos1
         sos2 = self._capabilities.sos2
         modelSOS = ModelSOS()
-        for block in all_blocks:
-            sos_con_list = block.active_components(SOSConstraint)
-        
-            if len(sos_con_list) > 0:
-                if not(sos1 or sos2 or sosn):
-                    raise Exception, "Solver does not support SOSConstraint declarations"
-
-            for con in sos_con_list.itervalues():
+        for block in pyomo_instance.all_blocks():
+            for con in active_subcomponents_generator(block,SOSConstraint):
                 level = con.sos_level()
                 if (level == 1 and not sos1) or (level == 2 and not sos2) or (level > 2 and not sosn):
-                    raise Exception, "Solver does not support SOS level %s constraints" % (level,)
+                    raise Exception("Solver does not support SOS level %s constraints" % (level,))
                 name = self._symbol_map.getSymbol( con, labeler ) 
                 masterIndex = con.sos_set_set()
                 if None in masterIndex:
@@ -420,72 +477,85 @@ class CPLEXDirect(OptSolver):
                     # A series of indexed constraints
                     for index in masterIndex:
                         modelSOS.count_constraint(self._symbol_map,labeler,con, name, level, index)
-        
+
         if modelSOS.sosType:
-            for key in modelSOS.sosType.iterkeys():
+            for key in modelSOS.sosType:
                 cplex_instance.SOS.add(type = modelSOS.sosType[key], \
                                        name = modelSOS.sosName[key], \
                                        SOS = [modelSOS.varnames[key], modelSOS.weights[key]] )
             used_sos_constraints = True
-        
-
-        # set the problem type based on the variable counts.
-        if (quadratic is True):
-            if (num_integer_variables > 0) or (num_binary_variables > 0) or (used_sos_constraints):
-                cplex_instance.set_problem_type(cplex_instance.problem_type.MIQP)
-            else:
-                cplex_instance.set_problem_type(cplex_instance.problem_type.QP)
-        elif (num_integer_variables > 0) or (num_binary_variables > 0) or (used_sos_constraints):
-            cplex_instance.set_problem_type(cplex_instance.problem_type.MILP)
-        else:
-            cplex_instance.set_problem_type(cplex_instance.problem_type.LP)
-
 
         cplex_instance.linear_constraints.add(lin_expr=expressions, senses=senses, rhs=rhss, range_values=range_values, names=names)
 
         for index in xrange(len(qexpressions)):
             cplex_instance.quadratic_constraints.add(lin_expr=qlinears[index], quad_expr=qexpressions[index], sense=qsenses[index], rhs=qrhss[index], name=qnames[index])
         
+        model_repn_suffix = pyomo_instance.subcomponent("repn")
+        if (model_repn_suffix is None) or (not model_repn_suffix.type() is Suffix) or (not model_repn_suffix.active is True):
+            raise ValueError("Unable to find an active Suffix with name 'repn' on block: %s" % (pyomo_instance.cname(True)))
 
         # transfer the objective.
-        active_objectives = pyomo_instance.active_components(Objective)
-        the_objective = active_objectives[active_objectives.keys()[0]]
-        if the_objective.sense == maximize:
-            cplex_instance.objective.set_sense(cplex_instance.objective.sense.maximize)
-        else:
-            cplex_instance.objective.set_sense(cplex_instance.objective.sense.minimize)
+        for obj_data in active_subcomponents_data_generator(pyomo_instance,Objective):
+            if obj_data.is_minimizing():
+                cplex_instance.objective.set_sense(cplex_instance.objective.sense.minimize)
+            else:
+                cplex_instance.objective.set_sense(cplex_instance.objective.sense.maximize)
 
-        cplex_instance.objective.set_name(the_objective.name)
+            cplex_instance.objective.set_name(obj_data.cname(True))
 
-        self._symbol_map.getSymbol(the_objective[None], labeler)
-        self._symbol_map.alias(the_objective[None],'__default_objective__')
+            self._symbol_map.getSymbol(obj_data, labeler)
+            self._symbol_map.alias(obj_data,'__default_objective__')
 
+            obj_repn = model_repn_suffix.getValue(obj_data)
 
-        if canonical_is_constant(the_objective[None].repn):
-            print ("Warning: Constant objective detected, replacing " + \
-                    "with a placeholder to prevent solver failure.")
+            if (isinstance(obj_repn, LinearCanonicalRepn) and (obj_repn.linear == None)) or canonical_is_constant(obj_repn):
+                print("Warning: Constant objective detected, replacing " + \
+                          "with a placeholder to prevent solver failure.")
 
-            cplex_instance.variables.add(lb=[1],ub=[1],names=["ONE_VAR_CONSTANT"])
-            objective_expression = [("ONE_VAR_CONSTANT",offset)]
-            cplex_instance.objective.set_linear(objective_expression)
-
-        else:
-            #Linear terms
-            if 1 in the_objective[None].repn:
-                objective_expression, offset = self._encode_constraint_body_linear(the_objective[None].repn, labeler, as_pairs=True) # how to deal with indexed objectives?
-                if offset != 0:
-                    cplex_instance.variables.add(lb=[1],ub=[1],names=["ONE_VAR_CONSTANT"])
-                    objective_expression.append(("ONE_VAR_CONSTANT",offset))
+                cplex_instance.variables.add(lb=[1],ub=[1],names=["ONE_VAR_CONSTANT"])
+                objective_expression = [("ONE_VAR_CONSTANT",offset)]
                 cplex_instance.objective.set_linear(objective_expression)
+                
+            else:
 
-            if 2 in the_objective[None].repn:
-                #Quadratic terms 
-                quadratic = True
-                objective_expression = self._encode_constraint_body_quadratic(the_objective[None].repn, labeler, as_triples=True, is_obj=2.0) # how to deal with indexed objectives?
-                cplex_instance.objective.set_quadratic_coefficients(objective_expression)
+                if isinstance(obj_repn, LinearCanonicalRepn):
+                    objective_expression, offset = self._encode_constraint_body_linear_specialized(obj_repn, labeler, as_pairs=True) # how to deal with indexed objectives?
+                    if offset != 0:
+                        cplex_instance.variables.add(lb=[1],ub=[1],names=["ONE_VAR_CONSTANT"])
+                        objective_expression.append(("ONE_VAR_CONSTANT",offset))
+                    cplex_instance.objective.set_linear(objective_expression)
+                else:
+                    #Linear terms
+                    if 1 in obj_repn:
+                        objective_expression, offset = self._encode_constraint_body_linear(obj_repn, labeler, as_pairs=True) # how to deal with indexed objectives?
+                        if offset != 0:
+                            cplex_instance.variables.add(lb=[1],ub=[1],names=["ONE_VAR_CONSTANT"])
+                            objective_expression.append(("ONE_VAR_CONSTANT",offset))
+                        cplex_instance.objective.set_linear(objective_expression)
 
+                    #Quadratic terms 
+                    if 2 in obj_repn:
+                        quadratic_objective = True
+                        objective_expression = self._encode_constraint_body_quadratic(obj_repn, labeler, as_triples=True, is_obj=2.0) # how to deal with indexed objectives?
+                        cplex_instance.objective.set_quadratic_coefficients(objective_expression)
+
+        # set the problem type based on the variable counts.
+        if (quadratic_objective is True) or (quadratic_constraints is True):
+            if (num_integer_variables > 0) or (num_binary_variables > 0) or (used_sos_constraints):
+                if quadratic_constraints is True:
+                    cplex_instance.set_problem_type(cplex_instance.problem_type.MIQCP)
+                else:
+                    cplex_instance.set_problem_type(cplex_instance.problem_type.MIQP)
+            else:
+                if quadratic_constraints is True:
+                    cplex_instance.set_problem_type(cplex_instance.problem_type.QCP)
+                else:
+                    cplex_instance.set_problem_type(cplex_instance.problem_type.QP)
+        elif (num_integer_variables > 0) or (num_binary_variables > 0) or (used_sos_constraints):
+            cplex_instance.set_problem_type(cplex_instance.problem_type.MILP)
+        else:
+            cplex_instance.set_problem_type(cplex_instance.problem_type.LP)
 
-        #cplex_instance.write("new.lp")
         self._active_cplex_instance = cplex_instance
 
     #
@@ -504,23 +574,21 @@ class CPLEXDirect(OptSolver):
             raise KeyError(
                 "CPLEX Warm Start Error: instance contains an object, %s, "
                 "that is not found in the symbol map.  Does the instance "
-                "match the provided symbol map?" % ( obj.name, ))
+                "match the provided symbol map?" % ( obj.cname(True), ))
 
         from coopr.pyomo.base import Var
 
-        labeler = CPLEXDirect._numeric_labeler()
+        labeler = NumericLabeler()
         # the iteration order is identical to that used in generating
         # the cplex instance, so all should be well.
         variable_names = []
         variable_values = []
-        for variable in instance.active_components(Var).values():
-            for index in variable: # TBD - change iteration style.
-                var = variable[index]
-                if (var.stale is True) or (var.fixed is True):
-                    continue
-                name = self._symbol_map.getSymbol(var, _error_labeler)
-                variable_names.append(name)
-                variable_values.append(var.value)
+        for var in active_subcomponents_data_generator(Var):
+            if (var.stale is True) or (var.fixed is True):
+                continue
+            name = self._symbol_map.getSymbol(var, _error_labeler)
+            variable_names.append(name)
+            variable_values.append(var.value)
 
         self._active_cplex_instance.MIP_starts.add([variable_names, variable_values],
                                                    self._active_cplex_instance.MIP_starts.effort_level.auto)
@@ -530,9 +598,9 @@ class CPLEXDirect(OptSolver):
     def _presolve(self, *args, **kwds):
         from coopr.pyomo.base.PyomoModel import Model
 
-        self.suffixes = kwds.pop( 'suffixes', [] )
+        self.suffixes = kwds.pop( 'suffixes', self.suffixes )
         self.tee = kwds.pop( 'tee', False )
-        self.keepFiles = kwds.pop( 'keepFiles', False )
+        self.keepfiles = kwds.pop( 'keepfiles', False )
         self.warm_start_solve = kwds.pop( 'warmstart', False )
 
         # Step 1: extract the pyomo instance from the input arguments,
@@ -541,14 +609,14 @@ class CPLEXDirect(OptSolver):
         if len(args) != 1:
             msg = "The CPLEXDirect plugin method '_presolve' must be supplied "\
                   "a single problem instance - %s were supplied"
-            raise ValueError, msg % len(args)
+            raise ValueError(msg % len(args))
 
         model = args[ 0 ]
         if not isinstance(model, Model):
             msg = "The problem instance supplied to the CPLEXDirect plugin " \
                   "method '_presolve' must be of type 'Model' - "\
                   "interface does not currently support file names"
-            raise ValueError, msg
+            raise ValueError(msg)
 
         # TBD-document.
         self._populate_cplex_instance(model)
@@ -574,14 +642,14 @@ class CPLEXDirect(OptSolver):
                 if len(args) != 1:
                     msg = "CPLEX _presolve method can only handle a single " \
                           "problem instance - %s were supplied"
-                    raise ValueError, msg % len(args)
+                    raise ValueError(msg % len(args))
 
                 if self._hasIntegerVariables(model) is True:
                     start_time = time.time()
                     self.warm_start(model)
                     end_time = time.time()
                     if self._report_timing is True:
-                        print "Warm start write time=%.2f seconds" % (end_time-start_time)
+                        print("Warm start write time=%.2f seconds" % (end_time-start_time))
 
     #
     # TBD
@@ -611,9 +679,9 @@ class CPLEXDirect(OptSolver):
             #other solvers set output using tee=True/False
             from sys import stdout
             self._active_cplex_instance.set_results_stream(stdout)
-        elif self.keepFiles == True:
+        elif self.keepfiles == True:
             log_file = pyutilib.services.TempfileManager.create_tempfile(suffix = '.cplex.log')
-            print "Solver log file: " + log_file
+            print("Solver log file: " + log_file)
             self._active_cplex_instance.set_results_stream(log_file)
             #Not sure why the following doesn't work. As a result, it's either stream output
             #or write a logfile, but not both.
@@ -622,6 +690,10 @@ class CPLEXDirect(OptSolver):
             self._active_cplex_instance.set_results_stream(None)
         self._active_cplex_instance.solve()
 
+        # FIXME: can we get a return code indicating if CPLEX had a
+        # significant failure?
+        return Bunch(rc=None, log=None)
+
     def _postsolve(self):
 
         # the only suffixes that we extract from CPLEX are
@@ -644,11 +716,18 @@ class CPLEXDirect(OptSolver):
                 extract_reduced_costs = True
                 flag=True
             if not flag:
-                raise RuntimeError,"***The CPLEX solver plugin cannot extract solution suffix="+suffix
+                raise RuntimeError("***The CPLEXDirect solver plugin cannot extract solution suffix="+suffix)
 
 
         instance = self._active_cplex_instance
 
+
+        if instance.get_problem_type() in [instance.problem_type.MILP,
+                                           instance.problem_type.MIQP,
+                                           instance.problem_type.MIQCP]:
+            extract_reduced_costs = False
+            extract_duals = False
+
         # Remove variables whose absolute value is smaller than CPLEX's epsilon from the results data
         #instance.cleanup()
 
@@ -680,7 +759,7 @@ class CPLEXDirect(OptSolver):
         results.solver.system_time = None
         results.solver.wallclock_time = None
         results.solver.termination_message = None
-
+        
         soln = Solution()
 
         #Get solution status -- for now, if CPLEX returns anything we don't recognize, mark as an error
@@ -706,17 +785,57 @@ class CPLEXDirect(OptSolver):
             num_variables = instance.variables.get_num()
             variable_names = instance.variables.get_names()
             variable_values = instance.solution.get_values()
-            for i in range(0,num_variables):
+            for i in xrange(num_variables):
                 variable_name = variable_names[i]
                 soln.variable[variable_name] = {"Value" : variable_values[i], "Id" : len(soln.variable)}
 
+            if extract_reduced_costs:
+                # get variable reduced costs
+                rc_values = instance.solution.get_reduced_costs()
+                for i in xrange(num_variables):
+                    soln.variable[variable_names[i]]["Rc"] = rc_values[i]
+
             if extract_duals:
+                # get duals (linear constraints only)
                 num_linear_constraints = instance.linear_constraints.get_num()
                 constraint_names = instance.linear_constraints.get_names()
                 dual_values = instance.solution.get_dual_values() 
-                for i in range(0,num_linear_constraints):
+                for i in xrange(num_linear_constraints):
                     soln.constraint[constraint_names[i]].dual = dual_values[i]
 
+                # CPLEX PYTHON API DOES NOT SUPPORT QUADRATIC DUAL COLLECTION
+
+            if extract_slacks:
+                # get linear slacks
+                num_linear_constraints = instance.linear_constraints.get_num()
+                constraint_names = instance.linear_constraints.get_names()
+                slack_values = instance.solution.get_linear_slacks() 
+                for i in xrange(num_linear_constraints):
+                    # if both U and L exist (i.e., a range constraint) then 
+                    # R_ = U-L
+                    R_ = instance.linear_constraints.get_range_values(i)
+                    if R_ == 0.0:
+                        soln.constraint[constraint_names[i]].slack = slack_values[i]
+                    else:
+                        # This is a range constraint for which cplex always returns the
+                        # value of f(x)-L. In the spirit of conforming with the other writer,
+                        # I will return the max (in absolute value) of L-f(x) and U-f(x)
+                        Ls_ = slack_values[i]
+                        Us_ = R_ - slack_values[i]
+                        if Us_ > Ls_:
+                            soln.constraint[constraint_names[i]].slack = Us_
+                        else:
+                            soln.constraint[constraint_names[i]].slack = -Ls_
+                
+                # get quadratic slacks
+                num_quadratic_constraints = instance.quadratic_constraints.get_num()
+                constraint_names = instance.quadratic_constraints.get_names()
+                slack_values = instance.solution.get_quadratic_slacks() 
+                for i in xrange(num_quadratic_constraints):
+                    # if both U and L exist (i.e., a range constraint) then 
+                    # R_ = U-L
+                    soln.constraint[constraint_names[i]].slack = slack_values[i]
+
             results.solution.insert(soln)
 
         self.results = results
