@@ -12,11 +12,7 @@ import logging
 import os
 import re
 
-from coopr.opt import *
-#from coopr.opt.base import ProblemFormat as PF, ResultsFormat as RF
-#from coopr.opt.results import ProblemSense as PS, SolverResults, \
-                              #SolutionStatus, SolverStatus, TerminationCondition
-from coopr.opt.solver import SystemCallSolver
+from six import iteritems
 
 from pyutilib.common import ApplicationError
 import pyutilib.component.core
@@ -24,27 +20,28 @@ from pyutilib.misc import Bunch, Options
 from pyutilib.services import register_executable, registered_executable
 from pyutilib.services import TempfileManager
 
+from coopr.opt import *
+from coopr.opt.base.solvers import _extract_version
+from coopr.opt.solver import SystemCallSolver
+
+
 Logger = logging.getLogger('coopr.plugins')
 
 glpk_file_flag=None
-
+_glpk_version = None
 def configure_glpk():
     global glpk_file_flag
+    global _glpk_version
     if glpk_file_flag is not None:
         return
     glpk_file_flag = False
     if registered_executable("glpsol") is None:
         return
     errcode, results = pyutilib.subprocess.run(
-        [registered_executable('glpsol').get_path(), "--version"])
+        [registered_executable('glpsol').get_path(), "--version"], timelimit=2)
     if errcode == 0:
-        lines = results.split('\n') 
-        if len(lines) == 0 or lines[0] == '':
-            return
-        m = re.search('(\d+)\.(\d+)', lines[0].split()[-1])
-        if m:
-            _version = tuple(int(i) for i in m.groups())
-            glpk_file_flag = _version >= (4,42)
+        _glpk_version = _extract_version(results)
+        glpk_file_flag = _glpk_version >= (4,42,0,0)
 
 
 # Not sure how better to get these constants, but pulled from GLPK
@@ -69,7 +66,7 @@ GLP_UNBND  = 6  # solution is unbounded
 class GLPK(OptSolver):
     """The GLPK LP/MIP solver"""
 
-    pyutilib.component.core.alias('glpk', doc='The GLPK MIP solver')
+    pyutilib.component.core.alias('glpk', doc='The GLPK LP/MIP solver')
 
     def __new__(cls, *args, **kwds):
         try:
@@ -121,14 +118,15 @@ class GLPKSHELL ( SystemCallSolver ):
           ProblemFormat.cpxlp: ResultsFormat.soln,
           ProblemFormat.mps:   ResultsFormat.soln,
         }
+        self.set_problem_format(ProblemFormat.cpxlp)
 
         # Note: Undefined capabilities default to 'None'
         self._capabilities = Options()
         self._capabilities.linear = True
         self._capabilities.integer = True
-        self._capabilities.sos1 = False
-        self._capabilities.sos2 = False
-        self._capabilities.sosn = False
+
+    def _default_results_format(self, prob_format):
+        return ResultsFormat.soln
 
     def executable ( self ):
         executable = registered_executable('glpsol')
@@ -141,6 +139,13 @@ class GLPKSHELL ( SystemCallSolver ):
 
         return executable.get_path()
 
+    def version(self):
+        """
+        Returns a tuple describing the solver executable version.
+        """
+        if _glpk_version is None:
+            return _extract_version('')
+        return _glpk_version
 
     def create_command_line ( self, executable, problem_files ):
         #
@@ -152,17 +157,10 @@ class GLPKSHELL ( SystemCallSolver ):
         #
         # Define solution file
         #
-        if self.soln_file is None:
-            self.soln_file = \
-                 TempfileManager.create_tempfile(suffix='.glpk.soln')
         self._glpfile = TempfileManager.create_tempfile(suffix='.glpk.glp')
         self._rawfile = TempfileManager.create_tempfile(suffix='.glpk.raw')
-
-        #
-        # Define results file
-        #
-        if self._results_format is None or self._results_format == ResultsFormat.soln:
-            self.results_file = self.soln_file
+        self.soln_file = self._rawfile
+        self.results_file = self.soln_file
 
         #
         # Define command line
@@ -250,10 +248,11 @@ class GLPKSHELL ( SystemCallSolver ):
         elif GLP_NOFEAS == status: return SolutionStatus.infeasible
         elif GLP_UNBND  == status: return SolutionStatus.unbounded
         elif GLP_UNDEF  == status: return SolutionStatus.other
-        raise RuntimeError, "Unknown solution status returned by GLPK solver"
+        raise RuntimeError("Unknown solution status returned by GLPK solver")
 
 
     def process_soln_file ( self, results ):
+        soln  = None
         pdata = self._glpfile
         psoln = self._rawfile
 
@@ -297,13 +296,19 @@ class GLPKSHELL ( SystemCallSolver ):
             prob.number_of_variables   = pcols
 
             extract_duals = False
+            extract_reduced_costs = False
             for suffix in self.suffixes:
+                flag = False
                 if re.match(suffix, "dual"):
-                    if self.is_integer:
-                        if suffix == 'dual':
-                            raise RuntimeError, 'Request for duals of an integer problem.'
-                    else:
+                    flag = True
+                    if not self.is_integer:
                         extract_duals = True
+                if re.match(suffix, "rc"):
+                    flag = True
+                    if not self.is_integer:
+                        extract_reduced_costs = True
+                if not flag:
+                    raise RuntimeError("***The GLPK solver plugin cannot extract solution suffix="+suffix)
 
             for line in f:
                 glp_line_count += 1
@@ -334,11 +339,12 @@ class GLPKSHELL ( SystemCallSolver ):
                     raise ValueError
 
             f.close()
-        except Exception, e:
+        except Exception:
+            e = sys.exc_info()[1]
             msg = "Error parsing solution description file, line %s: %s"
-            raise ValueError, msg % (glp_line_count, str(e))
-
+            raise ValueError(msg % (glp_line_count, str(e)))
 
+        range_duals = {}
         # Step 2: Make use of the GLPK's machine parseable format (--write) to
         #    collect solution variable and constraint values.
         raw_line_count = ' -- File not yet opened'
@@ -356,13 +362,11 @@ class GLPKSHELL ( SystemCallSolver ):
             else:
                 pstat, dstat, obj_val = f.readline().split()
                 dstat = float( dstat ) # dual status of basic solution.  Ignored.
-            #print "HERE", pstat, obj_val, self.is_integer
 
             pstat = float( pstat )       # fails if not a number; intentional
             obj_val = float( obj_val )   # fails if not a number; intentional
             soln_status = self._glpk_get_solution_status( pstat )
 
-            #print "HEREZ", soln_status
             if soln_status is SolutionStatus.infeasible:
                 solv.termination_condition = TerminationCondition.infeasible
 
@@ -406,10 +410,13 @@ class GLPKSHELL ( SystemCallSolver ):
                     rprim = float( rprim )
                     soln.constraint[ cname ].value = rprim
 
-                    #print "HERE", extract_duals, self.is_integer
-                    if extract_duals and not self.is_integer:
-                        rdual = float(rdual)
-                        soln.constraint[ cname ].dual  = rdual
+                    if extract_duals:
+                        if cname.startswith('c_'):
+                            soln.constraint[ cname ].dual  = float(rdual)
+                        elif cname.startswith('r_l_'):
+                            range_duals.setdefault(cname[4:],[0,0])[0] = float(rdual)
+                        elif cname.startswith('r_u_'):
+                            range_duals.setdefault(cname[4:],[0,0])[1] = float(rdual)
 
                 for nn in range( 1, pcols +1 ):
                     raw_line_count += 1
@@ -422,19 +429,28 @@ class GLPKSHELL ( SystemCallSolver ):
                     vname = variable_names[ nn ]
                     if 'ONE_VAR_CONSTANT' == vname: continue
                     cprim = float( cprim )
-                    soln.variable[ vname ] = {"Value" : cprim, "Id" : len(soln.variable)}
-
-                    #if extract_duals and not self.is_integer:
-                    #       cdual = float(cdual)
-                    #       soln.variable[ vname ].dual  = cdual
-                    #       GLPK reports a dual for variables, though Pyomo does not have
-                    #       a place to put them
+                    if extract_reduced_costs is False:
+                        soln.variable[ vname ] = {"Value" : cprim, "Id" : len(soln.variable)}
+                    else:
+                        soln.variable[ vname ] = {"Value" : cprim,
+                                                  "Id" : len(soln.variable),
+                                                  "Rc" : float(cdual)}
 
             f.close()
-        except Exception, e:
-            print e
+        except Exception:
+            print(sys.exc_info()[1])
             msg = "Error parsing solution data file, line %d" % raw_line_count
-            raise ValueError, msg
+            raise ValueError(msg)
+
+        if not soln is None:
+            # For the range constraints, supply only the dual with the largest
+            # magnitude (at least one should always be numericly zero)
+            scon = soln.Constraint
+            for key,(ld,ud) in iteritems(range_duals):
+                if abs(ld) > abs(ud):
+                    scon['r_l_'+key].dual = ld
+                else:
+                    scon['r_u_'+key].dual = ud
 
 
 #class MockGLPK(GLPK,mockmip.MockMIP):
