@@ -12,17 +12,20 @@
 import os
 import re
 import logging
+import sys
 
-from coopr.opt.base import *
-from coopr.opt.results import *
-from coopr.opt.solver import *
-import mockmip
 from pyutilib.common import ApplicationError
 from pyutilib.component.core import alias, PluginGlobals
 from pyutilib.misc import Bunch, Options
 from pyutilib.services import register_executable, registered_executable
 from pyutilib.services import TempfileManager
+import pyutilib.subprocess
 
+from coopr.opt.base import *
+from coopr.opt.base.solvers import _extract_version
+from coopr.opt.results import *
+from coopr.opt.solver import *
+from coopr.plugins.solvers.mockmip import MockMIP
 
 class GLPKSHELL_old(SystemCallSolver):
     """Shell interface to the GLPK LP/MIP solver"""
@@ -48,14 +51,15 @@ class GLPKSHELL_old(SystemCallSolver):
             ProblemFormat.cpxlp: ResultsFormat.soln,
             ProblemFormat.mps: ResultsFormat.soln,
         }
+        self.set_problem_format(ProblemFormat.cpxlp)
 
         # Note: Undefined capabilities default to 'None'
         self._capabilities = Options()
         self._capabilities.linear = True
         self._capabilities.integer = True
-        self._capabilities.sos1 = False
-        self._capabilities.sos2 = False
-        self._capabilities.sosn = False
+
+    def _default_results_format(self, prob_format):
+        return ResultsFormat.soln
 
     def executable(self):
         executable = registered_executable('glpsol')
@@ -68,6 +72,19 @@ class GLPKSHELL_old(SystemCallSolver):
 
         return executable.get_path()
 
+    def version(self):
+        """
+        Returns a tuple describing the solver executable version.
+        """
+        solver_exec = self.executable()
+        if solver_exec is None:
+            return _extract_version('')
+        errcode, results = pyutilib.subprocess.run(
+        [solver_exec, "--version"], timelimit=1)
+        if errcode == 0:
+            return _extract_version(results)
+        return _extract_version('')
+
     def create_command_line(self, executable, problem_files):
         #
         # Define log file
@@ -78,16 +95,8 @@ class GLPKSHELL_old(SystemCallSolver):
         #
         # Define solution file
         #
-        if self.soln_file is None:
-            self.soln_file = \
-                           TempfileManager.create_tempfile(suffix='.glpk.soln')
-
-        #
-        # Define results file
-        #
-        if self._results_format is None \
-           or self._results_format == ResultsFormat.soln:
-            self.results_file = self.soln_file
+        self.soln_file = TempfileManager.create_tempfile(suffix='.glpk.soln')
+        self.results_file = self.soln_file
 
         #
         # Define command line
@@ -237,16 +246,17 @@ class GLPKSHELL_old(SystemCallSolver):
                 extract_duals = True
                 flag = True
             if not flag:
-                raise RuntimeError, \
-                      ("***GLPK solver plugin cannot extract solution " + \
-                       "suffix='%s'") % (suffix)
+                raise RuntimeError(\
+                      "***GLPK solver plugin cannot extract solution " + \
+                       "suffix='%s'" % (suffix))
 
         lp_solution = True  # if false, we're dealing with a MIP!
         if not os.path.exists(self.soln_file):
             return
         soln = results.solution(0)
         INPUT = open(self.soln_file, "r")
-
+        
+        range_duals = {}
         try:
 
             state = 0  # 0=initial header, 1=constraints, 2=variables, -1=done
@@ -264,7 +274,7 @@ class GLPKSHELL_old(SystemCallSolver):
             # variable names and their value/bounds can be split across
             # multiple lines
             active_variable_name = ""
-
+            
             for line in INPUT:
                 tokens = re.split('[ \t]+', line.strip())
 
@@ -333,12 +343,12 @@ class GLPKSHELL_old(SystemCallSolver):
 
                         # sanity check - the indices should be in sequence.
                         if index != number_of_constraints_read:
-                            raise ValueError, \
+                            raise ValueError(\
                                   ("***ERROR: Unexpected constraint index " + \
                                    "encountered on line=%s; expected " + \
                                    "value=%s; actual value=%s") % \
                                    (line, str(number_of_consrtaints_read),
-                                    str(index))
+                                    str(index)))
                     else:
                         index = None
                         activity = None
@@ -369,7 +379,6 @@ class GLPKSHELL_old(SystemCallSolver):
                         if lp_solution is True:
                             state_string = line[20:22].strip()
                             marginal_string = line[65:78].strip()
-                            #print "HERE '%s' '%s' '%s'" % (state_string, upper_bound_string, marginal_string)
                             if (activity_string != "< eps") and (len(activity_string) > 0):
                                 activity = eval(activity_string)
                             else:
@@ -405,7 +414,7 @@ class GLPKSHELL_old(SystemCallSolver):
                             number_of_constraints_read = number_of_constraints_read + 1
                             # sanity check - the indices should be in sequence.
                             if index != number_of_constraints_read:
-                                raise ValueError,"***ERROR: Unexpected constraint index encountered on line="+line+"; expected value="+str(number_of_constraints_read)+"; actual value="+str(index)
+                                raise ValueError("***ERROR: Unexpected constraint index encountered on line="+line+"; expected value="+str(number_of_constraints_read)+"; actual value="+str(index))
 
                         if lp_solution is True:
                             # GLPK doesn't report slacks directly.
@@ -415,10 +424,15 @@ class GLPKSHELL_old(SystemCallSolver):
                             elif (state_string == "NS") or (state_string == "NL") or (state_string == "NU"):
                                 constraint_dual = marginal
                             else:
-                                raise ValueError, "Unknown status="+tokens[0]+" encountered for constraint="+active_constraint_name+" in line="+line+" of solution file="+self.soln_file
+                                raise ValueError("Unknown status="+tokens[0]+" encountered for constraint="+active_constraint_name+" in line="+line+" of solution file="+self.soln_file)
 
                             if extract_duals is True:
-                                soln.constraint[constraint_name].dual = constraint_dual
+                                if constraint_name.startswith('c_'):
+                                    soln.constraint[constraint_name].dual  = float(constraint_dual)
+                                elif constraint_name.startswith('r_l_'):
+                                    range_duals.setdefault(constraint_name[4:],[0,0])[0] = float(constraint_dual)
+                                elif constraint_name.startswith('r_u_'):
+                                    range_duals.setdefault(constraint_name[4:],[0,0])[1] = float(constraint_dual)
 
                         else:
                             # there isn't anything interesting to do with constraints in the MIP case.
@@ -443,7 +457,7 @@ class GLPKSHELL_old(SystemCallSolver):
 
                         # sanity check - the indices should be in sequence.
                         if index != number_of_variables_read:
-                            raise ValueError,"***ERROR: Unexpected variable index encountered on line="+line+"; expected value="+str(number_of_variables_read)+"; actual value="+str(index)
+                            raise ValueError("***ERROR: Unexpected variable index encountered on line="+line+"; expected value="+str(number_of_variables_read)+"; actual value="+str(index))
 
                     else:
 
@@ -514,7 +528,7 @@ class GLPKSHELL_old(SystemCallSolver):
                             number_of_variables_read = number_of_variables_read + 1
                             # sanity check - the indices should be in sequence.
                             if index != number_of_variables_read:
-                                raise ValueError,"***ERROR: Unexpected variable index encountered on line="+line+"; expected value="+str(number_of_variables_read)+"; actual value="+str(index)
+                                raise ValueError("***ERROR: Unexpected variable index encountered on line="+line+"; expected value="+str(number_of_variables_read)+"; actual value="+str(index))
 
                         if lp_solution is True:
                             # the "activity" column always specifies the variable value.
@@ -528,7 +542,7 @@ class GLPKSHELL_old(SystemCallSolver):
                                 # NS = non-basic fixed variable
                                 variable_value = activity
                             else:
-                                raise ValueError, "Unknown status="+state_string+" encountered for variable="+variable_name+" in the following line of the GLPK solution file="+self.soln_file+":\n"+line
+                                raise ValueError("Unknown status="+state_string+" encountered for variable="+variable_name+" in the following line of the GLPK solution file="+self.soln_file+":\n"+line)
 
                             variable = soln.variable[variable_name] = {"Value" : variable_value, "Id" : len(soln.variable)}
                         else:
@@ -543,13 +557,24 @@ class GLPKSHELL_old(SystemCallSolver):
 
             INPUT.close()
 
-        except ValueError, msg:
+        except ValueError:
+            msg = sys.exc_info()[1]
             INPUT.close()
-            raise RuntimeError, msg
-        except Exception, msg:
+            raise RuntimeError(msg)
+        except Exception:
+            msg = sys.exc_info()[1]
             INPUT.close()
             raise
-            #raise RuntimeError, "Unexpected input encountered in GLPK solution file, yielding parse failure; line="+str(line)+"\nException: "+str(msg)
+
+        # For the range constraints, supply only the dual with the largest
+        # magnitude (at least one should always be numericly zero)
+        scon = soln.Constraint
+        for key,(ld,ud) in range_duals.items():
+            if abs(ld) > abs(ud):
+                scon['r_l_'+key].dual = ld
+            else:
+                scon['r_u_'+key].dual = ud
+
         #
         if soln.status is SolutionStatus.optimal:
             soln.gap = 0.0
@@ -574,7 +599,7 @@ class GLPKSHELL_old(SystemCallSolver):
                                eval(soln.objective(0))
 
 
-class MockGLPK(GLPKSHELL_old,mockmip.MockMIP):
+class MockGLPK(GLPKSHELL_old,MockMIP):
     """A Mock GLPK solver used for testing
     """
 
@@ -585,21 +610,21 @@ class MockGLPK(GLPKSHELL_old,mockmip.MockMIP):
             GLPKSHELL_old.__init__(self, **kwds)
         except ApplicationError: #pragma:nocover
             pass                        #pragma:nocover
-        mockmip.MockMIP.__init__(self,"glpk")
+        MockMIP.__init__(self,"glpk")
 
     def available(self, exception_flag=True):
         return GLPKSHELL_old.available(self,exception_flag)
 
     def create_command_line(self,executable,problem_files):
         command = GLPKSHELL_old.create_command_line(self,executable,problem_files)
-        mockmip.MockMIP.create_command_line(self,executable,problem_files)
+        MockMIP.create_command_line(self,executable,problem_files)
         return command
 
     def executable(self):
-        return mockmip.MockMIP.executable(self)
+        return MockMIP.executable(self)
 
     def _execute_command(self,cmd):
-        return mockmip.MockMIP._execute_command(self,cmd)
+        return MockMIP._execute_command(self,cmd)
 
     def _convert_problem(self,args,pformat,valid_pformats):
         if pformat in [ProblemFormat.mps,ProblemFormat.cpxlp]:
