@@ -11,20 +11,29 @@
 import os
 import sys
 import re
-from coopr.opt.base import *
-from coopr.opt.results import *
-from coopr.opt.solver import *
 import logging
-import mockmip
-import pyutilib.services
-import pyutilib.common
-import pyutilib.misc
-import pyutilib.component.core
 import string
 import re
 import xml.dom.minidom
 import time
 
+import pyutilib.services
+import pyutilib.common
+import pyutilib.misc
+import pyutilib.component.core
+
+from coopr.opt.base import *
+from coopr.opt.base.solvers import _extract_version
+from coopr.opt.results import *
+from coopr.opt.solver import *
+from coopr.pyomo.base.blockutil import has_discrete_variables
+
+from six import iteritems, StringIO
+
+try:
+    unicode
+except:
+    basestring = str
 
 class GUROBI(OptSolver):
     """The GUROBI LP/MIP solver
@@ -88,17 +97,21 @@ class GUROBISHELL(ILMLicensedSystemCallSolver):
         self._valid_result_formats={}
         self._valid_result_formats[ProblemFormat.cpxlp] = [ResultsFormat.soln]
         self._valid_result_formats[ProblemFormat.mps] = [ResultsFormat.soln]
+        self.set_problem_format(ProblemFormat.cpxlp)
 
         # Note: Undefined capabilities default to 'None'
         self._capabilities = pyutilib.misc.Options()
         self._capabilities.linear = True
-        self._capabilities.quadratic = True
+        self._capabilities.quadratic_objective = True
+        self._capabilities.quadratic_constraint = True
         self._capabilities.integer = True
         self._capabilities.sos1 = True
         self._capabilities.sos2 = True
 
-    def warm_start_capable(self):
+    def _default_results_format(self, prob_format):
+        return ResultsFormat.soln
 
+    def warm_start_capable(self):
         return True
 
     #
@@ -129,13 +142,17 @@ class GUROBISHELL(ILMLicensedSystemCallSolver):
                     if (var.stale is True) or (var.fixed is True):
                         continue
                     name = self._symbol_map.getSymbol(var, _error_labeler)
-                    print >>mst_file, name, var.value
+                    mst_file.write("%s %s\n" % (name, str(var.value)))
 
         mst_file.close()
 
     # over-ride presolve to extract the warm-start keyword, if specified.
     def _presolve(self, *args, **kwds):
 
+        # create a context in the temporary file manager for 
+        # this plugin - is "pop"ed in the _postsolve method.
+        pyutilib.services.TempfileManager.push()
+
         # if the first argument is a string (representing a filename),
         # then we don't have an instance => the solver is being applied
         # to a file.
@@ -147,7 +164,7 @@ class GUROBISHELL(ILMLicensedSystemCallSolver):
         # mechanism, invoked warm_start() with a instance to create the warm start file.
         if (self.warm_start_solve is True) and (isinstance(args[0],basestring) is True):
             pass # we assume the user knows what they are doing...
-        elif (self.warm_start_solve is True) and (isinstance(args[0], basestring) is False) and (args[0].has_discrete_variables() is True):
+        elif (self.warm_start_solve is True) and (isinstance(args[0], basestring) is False) and (has_discrete_variables(args[0]) is True):
            # assign the name of the warm start file *before* calling the base class
            # presolve - the base class method ends up creating the command line,
            # and the warm start file-name is (obviously) needed there.
@@ -172,12 +189,12 @@ class GUROBISHELL(ILMLicensedSystemCallSolver):
                         "GUROBI _presolve method can only handle a single "
                         "problem instance - %s were supplied" % (len(args),))
 
-                if args[0].has_discrete_variables() is True:
+                if has_discrete_variables(args[0]) is True:
                     start_time = time.time()
                     self.warm_start(args[0])
                     end_time = time.time()
                     if self._report_timing is True:
-                        print "Warm start write time=%.2f seconds" % (end_time-start_time)
+                        print("Warm start write time=%.2f seconds" % (end_time-start_time))
 
     def executable(self):
 
@@ -191,6 +208,24 @@ class GUROBISHELL(ILMLicensedSystemCallSolver):
             return None
         return executable.get_path()
 
+    def version(self):
+        """
+        Returns a tuple describing the solver executable version.
+        """
+        solver_exec = self.executable()
+        if solver_exec is None:
+            return _extract_version('')
+        outname = pyutilib.services.TempfileManager.create_tempfile(suffix = '.gurobi.version')
+        with open(outname,'w') as f:
+            # **Note, adding a 'timelimit' keyword here results in empty output for some reason
+            results = pyutilib.subprocess.run( [solver_exec], stdin='from gurobipy import *; print gurobi.version(); exit()', ostream=f) 
+        tmp = None
+        with open(outname,'r') as f:
+            tmp = tuple(eval(f.read().strip()))
+        while(len(tmp) < 4):
+            tmp += (0,)
+        return tmp[:4]
+
     def create_command_line(self,executable,problem_files):
 
         #
@@ -239,15 +274,15 @@ class GUROBISHELL(ILMLicensedSystemCallSolver):
 
         # dump the script and warm-start file names for the
         # user if we're keeping files around.
-        if self.keepFiles:
+        if self.keepfiles:
             script_fname = pyutilib.services.TempfileManager.create_tempfile(suffix = '.gurobi.script')
             script_file = open(script_fname, 'w')
             script_file.write( script )
             script_file.close()
 
-            print "Solver script file: '%s'" % script_fname
+            print("Solver script file: '%s'" % script_fname)
             if (self.warm_start_solve is True) and (self.warm_start_file_name is not None):
-                print "Solver warm-start file: " + self.warm_start_file_name
+                print("Solver warm-start file: " + self.warm_start_file_name)
 
         #
         # Define command line
@@ -285,12 +320,13 @@ class GUROBISHELL(ILMLicensedSystemCallSolver):
         section = 0 # unknown
 
         solution_seen = False
-
+        
+        range_duals = {}
+        range_slacks = {}
         INPUT = open(self.soln_file,"r")
         for line in INPUT:
             line = line.strip()
             tokens = [token.strip() for token in line.split(":")]
-
             if (tokens[0] == 'section'):
                 if (tokens[1] == 'problem'):
                     section = 1
@@ -316,11 +352,23 @@ class GUROBISHELL(ILMLicensedSystemCallSolver):
                         else:
                             results.problem.lower_bound = float(tokens[1])
                     elif (tokens[0] == 'constraintdual'):
-                        if tokens[1] != "c_e_ONE_VAR_CONSTANT":
-                            soln_constraints[tokens[1]].dual = float(tokens[2])
+                        name = tokens[1]
+                        if name != "c_e_ONE_VAR_CONSTANT":
+                            if name.startswith('c_'):
+                                soln_constraints[tokens[1]].dual = float(tokens[2])
+                            elif name.startswith('r_l_'):
+                                range_duals.setdefault(name[4:],[0,0])[0] = float(tokens[2])
+                            elif name.startswith('r_u_'):
+                                range_duals.setdefault(name[4:],[0,0])[1] = float(tokens[2])
                     elif (tokens[0] == 'constraintslack'):
-                        if tokens[1] != "c_e_ONE_VAR_CONSTANT":
-                            soln_constraints[tokens[1]].slack = float(tokens[2])
+                        name = tokens[1]
+                        if name != "c_e_ONE_VAR_CONSTANT":
+                            if name.startswith('c_'):
+                                soln_constraints[tokens[1]].slack = float(tokens[2])
+                            elif name.startswith('r_l_'):
+                                range_slacks.setdefault(name[4:],[0,0])[0] = float(tokens[2])
+                            elif name.startswith('r_u_'):
+                                range_slacks.setdefault(name[4:],[0,0])[1] = float(tokens[2])
                     elif (tokens[0] == 'varrc'):
                         if tokens[1] != "ONE_VAR_CONSTANT":
                             soln_variables[tokens[1]]["Rc"] = float(tokens[2])
@@ -355,9 +403,54 @@ class GUROBISHELL(ILMLicensedSystemCallSolver):
 
         INPUT.close()
 
+        # For the range constraints, supply only the dual with the largest
+        # magnitude (at least one should always be numericly zero)
+        scon = soln.Constraint
+        for key,(ld,ud) in iteritems(range_duals):
+            if abs(ld) > abs(ud):
+                scon['r_l_'+key].dual = ld
+            else:
+                scon['r_u_'+key].dual = ud
+        # slacks
+        for key,(ls,us) in iteritems(range_slacks):
+            if abs(ls) > abs(us):
+                scon['r_l_'+key].slack = ls
+            else:
+                scon['r_u_'+key].slack = us
+
+
         if solution_seen is True:
             results.solution.insert(soln)
 
+    def _postsolve(self):
+
+        # take care of the annoying GUROBI log file in the current directory.
+        # this approach doesn't seem overly efficient, but python os module functions 
+        # doesn't accept regular expression directly.
+        filename_list = os.listdir(".")
+        for filename in filename_list:
+            # IMPT: trap the possible exception raised by the file not existing.
+            #       this can occur in pyro environments where > 1 workers are
+            #       running GUROBI, and were started from the same directory.
+            #       these logs don't matter anyway (we redirect everything),
+            #       and are largely an annoyance.
+            try:
+                if  re.match('gurobi\.log', filename) != None:
+                    os.remove(filename)
+            except OSError:
+                pass
+
+        # let the base class deal with returning results.
+        results = ILMLicensedSystemCallSolver._postsolve(self)
+
+        # finally, clean any temporary files registered with the temp file
+        # manager, created populated *directly* by this plugin. does not
+        # include, for example, the execution script. but does include 
+        # the warm-start file. 
+        pyutilib.services.TempfileManager.pop(remove=not self.keepfiles)
+
+        return results
+
 if sys.platform == 'win32':
     pyutilib.services.register_executable(name='gurobi.bat')
 else:
