@@ -9,10 +9,13 @@
 #  _________________________________________________________________________
 
 
-from gurobipy import *
 import re
 import string
 
+from gurobipy import *
+
+GUROBI_VERSION = gurobi.version()
+
 # NOTE: this function / module is independent of Coopr, and only relies on the
 #       GUROBI python bindings. consequently, nothing in this function should
 #       throw an exception that is expected to be handled by Coopr - it won't be.
@@ -37,14 +40,24 @@ def gurobi_run(model_file, warmstart_file, soln_file, mipgap, options, suffixes)
             extract_reduced_costs = True
             flag=True
         if not flag:
-            print "***The GUROBI solver plugin cannot extract solution suffix="+suffix
+            print("***The GUROBI solver plugin cannot extract solution suffix="+suffix)
             return
 
     # Load the lp model
     model = read(model_file)
 
+    # if the use wants to extract duals or reduced costs and the
+    # model has quadratic constraints then we need to set the 
+    # QCPDual param to 1 (which apparently makes the solve more
+    # expensive in the quadratic case). If we do not set this param
+    # and and we attempt to access these suffixes in the solution
+    # printing the module will crash (when we have a QCP)
+    if GUROBI_VERSION[0] >= 5:
+        if (extract_reduced_costs is True) or (extract_duals is True):
+            model.setParam(GRB.Param.QCPDual,1)
+
     if model is None:
-        print "***The GUROBI solver plugin failed to load the input LP file="+soln_file
+        print("***The GUROBI solver plugin failed to load the input LP file="+soln_file)
         return
 
     if warmstart_file is not None:
@@ -134,9 +147,9 @@ def gurobi_run(model_file, warmstart_file, soln_file, mipgap, options, suffixes)
     solnfile = open(soln_file, "w+")
 
     # write the information required by results.problem
-    print >>solnfile, "section:problem"
+    solnfile.write("section:problem\n")
     name = model.getAttr(GRB.Attr.ModelName)
-    print >>solnfile, "name:",name
+    solnfile.write("name: "+name+'\n')
 
     sense = model.getAttr(GRB.Attr.ModelSense)
 
@@ -144,81 +157,89 @@ def gurobi_run(model_file, warmstart_file, soln_file, mipgap, options, suffixes)
     # this line fails for some reason so set the value to unknown
     try:
         bound = model.getAttr(GRB.Attr.ObjBound)
-    except Exception, e:
+    except Exception:
         if term_cond == 'optimal':
             bound = obj_value
         else:
             bound = None
 
     if (sense < 0):
-        print >>solnfile, "sense:maximize"
+        solnfile.write("sense:maximize\n")
         if bound is None:
-            print >>solnfile, "upper_bound:",float('infinity')
+            solnfile.write("upper_bound: %f\n" % float('infinity'))
         else:
-            print >>solnfile, "upper_bound:",bound
+            solnfile.write("upper_bound: %s\n" % str(bound))
     else:
-        print >>solnfile, "sense:minimize"
+        solnfile.write("sense:minimize\n")
         if bound is None:
-            print >>solnfile, "lower_bound:",float('-infinity')
+            solnfile.write("lower_bound: %f\n" % float('-infinity'))
         else:
-            print >>solnfile, "lower_bound:",bound
+            solnfile.write("lower_bound: %s\n" % str(bound))
 
     # TODO: Get the number of objective functions from GUROBI
     n_objs = 1
-    print >>solnfile, "number_of_objectives:", n_objs
+    solnfile.write("number_of_objectives: %d\n" % n_objs)
 
     cons = model.getConstrs()
-    print >>solnfile, "number_of_constraints:",len(cons)
+    qcons = []
+    if GUROBI_VERSION[0] >= 5:
+        qcons = model.getQConstrs()
+    solnfile.write("number_of_constraints: %d\n" % (len(cons)+len(qcons)+model.NumSOS,))
 
     vars = model.getVars()
-    print >>solnfile, "number_of_variables:",len(vars)
+    solnfile.write("number_of_variables: %d\n" % len(vars))
 
     n_binvars = model.getAttr(GRB.Attr.NumBinVars)
-    print >>solnfile, "number_of_binary_variables:", n_binvars
+    solnfile.write("number_of_binary_variables: %d\n" % n_binvars)
 
     n_intvars = model.getAttr(GRB.Attr.NumIntVars)
-    print >>solnfile, "number_of_integer_variables:", n_intvars
+    solnfile.write("number_of_integer_variables: %d\n" % n_intvars)
 
-    print >>solnfile, "number_of_continuous_variables:", len(vars)-n_intvars
+    solnfile.write("number_of_continuous_variables: %d\n" % (len(vars)-n_intvars,))
 
-    print >>solnfile, "number_of_nonzeros:",model.getAttr(GRB.Attr.NumNZs)
+    solnfile.write("number_of_nonzeros: %d\n" % model.getAttr(GRB.Attr.NumNZs))
 
     # write out the information required by results.solver
-    print >>solnfile, "section:solver"
+    solnfile.write("section:solver\n")
 
-    print >>solnfile, 'status:', status
-    print >>solnfile, 'return_code:', return_code
-    print >>solnfile, 'message:',message
-    print >>solnfile, 'user_time:', model.getAttr(GRB.Attr.Runtime)
-    print >>solnfile, 'system_time:', str(0.0)
-    print >>solnfile, 'termination_condition:', term_cond
-    print >>solnfile, 'termination_message:', message
+    solnfile.write('status: %s\n' % status)
+    solnfile.write('return_code: %s\n' % str(return_code))
+    solnfile.write('message: %s\n' % message)
+    solnfile.write('user_time: %s\n' % str(model.getAttr(GRB.Attr.Runtime)))
+    solnfile.write('system_time: %s\n' % str(0.0))
+    solnfile.write('termination_condition: %s\n' % term_cond)
+    solnfile.write('termination_message: %s\n' % message)
 
     is_discrete = False
     if (model.getAttr(GRB.Attr.IsMIP)):
         is_discrete = True
 
     if (term_cond == 'optimal') or (model.getAttr(GRB.Attr.SolCount) >= 1):
-        print >>solnfile, 'section:solution'
-        print >>solnfile, 'status:optimal'
-        print >>solnfile, 'message:',message
-        print >>solnfile, 'objective:',obj_value
-        print >>solnfile, 'gap:',0.
+        solnfile.write('section:solution\n')
+        solnfile.write('status:optimal\n')
+        solnfile.write('message: %s\n' % message)
+        solnfile.write('objective: %s\n' % str(obj_value))
+        solnfile.write('gap: 0.0\n')
 
         for var in vars:
-            print >>solnfile, 'var:',var.getAttr(GRB.Attr.VarName), ":",var.getAttr(GRB.Attr.X)
+            solnfile.write('var: %s : %s\n' % (str(var.getAttr(GRB.Attr.VarName)), str(var.getAttr(GRB.Attr.X))))
 
         if (is_discrete is False) and (extract_reduced_costs is True):
-            print "RC=",var.getAttr(GRB.Attr.RC)
-            print >>solnfile, 'varrc:',var.getAttr(GRB.Attr.VarName), ":",var.getAttr(GRB.Attr.RC)
+            for var in vars:
+                solnfile.write('varrc: %s : %s\n' % (str(var.getAttr(GRB.Attr.VarName)), str(var.getAttr(GRB.Attr.RC))))
 
         if (is_discrete is False) and (extract_duals is True):
             for con in cons:
                # Pi attributes in Gurobi are the constraint duals
-                print >>solnfile, "constraintdual:",con.getAttr(GRB.Attr.ConstrName),":",con.getAttr(GRB.Attr.Pi)
+                solnfile.write("constraintdual: %s : %s\n" % (str(con.getAttr(GRB.Attr.ConstrName)), str(con.getAttr(GRB.Attr.Pi))))
+            for con in qcons:
+                # QCPI attributes in Gurobi are the constraint duals
+                solnfile.write("constraintdual: %s : %s\n" % (str(con.getAttr(GRB.Attr.QCName)), str(con.getAttr(GRB.Attr.QCPi))))
 
-        if (is_discrete is True) and (extract_slacks is True):
+        if (extract_slacks is True):
             for con in cons:
-                print >>solnfile, "constraintslack:",con.getAttr(GRB.Attr.ConstrName),":",con.getAttr(GRB.Attr.Slack)
+                solnfile.write("constraintslack: %s : %s\n" % (con.getAttr(GRB.Attr.ConstrName), str(con.getAttr(GRB.Attr.Slack))))
+            for con in qcons:
+                solnfile.write("constraintslack: %s : %s\n" % (con.getAttr(GRB.Attr.QCName), str(con.getAttr(GRB.Attr.QCSlack))))
 
     solnfile.close()
