@@ -11,20 +11,23 @@
 
 import os
 import re
-from coopr.opt.base import *
-from coopr.opt.results import *
-from coopr.opt.solver import *
-import mockmip
-import pyutilib.services
-import pyutilib.common
-import pyutilib.misc
-import pyutilib.component.core
 import string
 import re
 import xml.dom.minidom
 import time
 import logging
 
+import pyutilib.services
+import pyutilib.common
+import pyutilib.misc
+import pyutilib.component.core
+
+from coopr.opt.base import *
+from coopr.opt.base.solvers import _extract_version
+from coopr.opt.results import *
+from coopr.opt.solver import *
+from coopr.plugins.solvers.mockmip import MockMIP
+
 
 class XPRESS(OptSolver):
     """The XPRESS LP/MIP solver
@@ -73,6 +76,7 @@ class XPRESS_shell(ILMLicensedSystemCallSolver):
         self._valid_result_formats={}
         self._valid_result_formats[ProblemFormat.cpxlp] = [ResultsFormat.soln]
         self._valid_result_formats[ProblemFormat.mps] = [ResultsFormat.soln]
+        self.set_problem_format(ProblemFormat.cpxlp)
 
         #
         # Cache the problem type - LP or MIP. Xpress needs to know this
@@ -82,11 +86,15 @@ class XPRESS_shell(ILMLicensedSystemCallSolver):
         # Note: Undefined capabilities default to 'None'
         self._capabilities = pyutilib.misc.Options()
         self._capabilities.linear = True
-        self._capabilities.quadratic = True
+        self._capabilities.quadratic_objective = True
+        self._capabilities.quadratic_constraint = True
         self._capabilities.integer = True
         self._capabilities.sos1 = True
         self._capabilities.sos2 = True
 
+    def _default_results_format(self, prob_format):
+        return ResultsFormat.soln
+
     #
     # we haven't reached this point quite yet.
     #
@@ -103,6 +111,14 @@ class XPRESS_shell(ILMLicensedSystemCallSolver):
             return None
         return executable.get_path()
 
+    # TODO: If anyone can get their hands on a working 'optimizer' executable
+    #       we could add a custom version method
+    def version(self):
+        """
+        Returns a tuple describing the solver executable version.
+        """
+        return _extract_version('')
+
     def create_command_line(self, executable, problem_files):
 
         #
@@ -116,14 +132,7 @@ class XPRESS_shell(ILMLicensedSystemCallSolver):
         # As indicated above, contains (in XML) both the solution and solver status.
         #
         self.soln_file = pyutilib.services.TempfileManager.create_tempfile(suffix = '.xpress.wrtsol')
-
-        #
-        # Define results file
-        #
-        if self._results_format is None or self._results_format == ResultsFormat.soln:
-            self.results_file = self.soln_file
-        elif self._results_format == ResultsFormat.sol:
-            self.results_file = self.sol_file
+        self.results_file = self.soln_file
 
         #
         # Write the XPRESS execution script
@@ -158,24 +167,19 @@ class XPRESS_shell(ILMLicensedSystemCallSolver):
 
         # dump the script and warm-start file names for the
         # user if we're keeping files around.
-        if self.keepFiles:
+        if self.keepfiles:
             script_fname = pyutilib.services.TempfileManager.create_tempfile(suffix = '.xpress.script')
             tmp = open(script_fname,'w')
             tmp.write(script)
             tmp.close()
             
-            print "Solver script file=" + script_fname
+            print("Solver script file=" + script_fname)
             if (self.warm_start_solve is True) and (self.warm_start_file_name is not None):
-                print "Solver warm-start file=" + self.warm_start_file_name
+                print("Solver warm-start file=" + self.warm_start_file_name)
 
         #
         # Define command line
         #
-        # JDS: I am not sure why we did this test... mostly because if
-        # it ever returned false, 'proc' was never initialized, resulting
-        # in an exception.
-        #if self._problem_format in [ProblemFormat.cpxlp, ProblemFormat.mps]:
-        #    proc = self._timer + " " + self.executable()
         cmd = [self.executable()]
         if self._timer:
             cmd.insert(0, self._timer)
@@ -294,6 +298,9 @@ class XPRESS_shell(ILMLicensedSystemCallSolver):
         extract_duals = False
         extract_slacks = False
         extract_reduced_costs = False
+        extract_rc = False
+        extract_lrc = False
+        extract_urc = False
         for suffix in self.suffixes:
             flag=False
             if re.match(suffix,"dual"):
@@ -304,9 +311,18 @@ class XPRESS_shell(ILMLicensedSystemCallSolver):
                 flag=True
             if re.match(suffix,"rc"):
                 extract_reduced_costs = True
+                extract_rc = True
+                flag=True
+            if re.match(suffix,"lrc"):
+                extract_reduced_costs = True
+                extract_lrc = True
+                flag=True
+            if re.match(suffix,"urc"):
+                extract_reduced_costs = True
+                extract_urc = True
                 flag=True
             if not flag:
-                raise RuntimeError,"***The xpress solver plugin cannot extract solution suffix="+suffix
+                raise RuntimeError("***The xpress solver plugin cannot extract solution suffix="+suffix)
 
         if not os.path.exists(self.soln_file):
             return
@@ -342,18 +358,21 @@ class XPRESS_shell(ILMLicensedSystemCallSolver):
                     variable = soln_variable[variable_name] = {"Value" : float(variable_value), "Id" : len(soln_variable)}
                     if (variable_reduced_cost is not None) and (extract_reduced_costs is True):
                         try:
-                            variable["Rc"] = float(variable_reduced_cost)
+                            if extract_rc is True:
+                                variable["Rc"] = float(variable_reduced_cost)
                             if variable_status is not None:
-                                if variable_status == "LL":
-                                    variable["Lrc"] = float(variable_reduced_cost)
-                                else:
-                                    variable["Lrc"] = 0.0
-                                if variable_status == "UL":
-                                    variable["Urc"] = float(variable_reduced_cost)
-                                else:
-                                    variable["Urc"] = 0.0
+                                if extract_lrc is True:
+                                    if variable_status == "LL":
+                                        variable["Lrc"] = float(variable_reduced_cost)
+                                    else:
+                                        variable["Lrc"] = 0.0
+                                if extract_urc is True:
+                                    if variable_status == "UL":
+                                        variable["Urc"] = float(variable_reduced_cost)
+                                    else:
+                                        variable["Urc"] = 0.0
                         except:
-                            raise ValueError, "Unexpected reduced-cost value="+str(variable_reduced_cost)+" encountered for variable="+variable_name
+                            raise ValueError("Unexpected reduced-cost value="+str(variable_reduced_cost)+" encountered for variable="+variable_name)
                         
             else:
                 
@@ -378,7 +397,7 @@ class XPRESS_shell(ILMLicensedSystemCallSolver):
             results.solution.insert(soln)
         solution_file.close()
 
-class MockXPRESS(XPRESS_shell,mockmip.MockMIP):
+class MockXPRESS(XPRESS_shell,MockMIP):
     """A Mock XPRESS solver used for testing
     """
 
@@ -389,21 +408,21 @@ class MockXPRESS(XPRESS_shell,mockmip.MockMIP):
             XPRESS_shell.__init__(self, **kwds)
         except pyutilib.common.ApplicationError: #pragma:nocover
             pass                                 #pragma:nocover
-        mockmip.MockMIP.__init__(self,"cplex")
+        MockMIP.__init__(self,"cplex")
 
     def available(self, exception_flag=True):
         return XPRESS_shell.available(self,exception_flag)
 
     def create_command_line(self,executable,problem_files):
         command = XPRESS_shell.create_command_line(self,executable,problem_files)
-        mockmip.MockMIP.create_command_line(self,executable,problem_files)
+        MockMIP.create_command_line(self,executable,problem_files)
         return command
 
     def executable(self):
-        return mockmip.MockMIP.executable(self)
+        return MockMIP.executable(self)
 
     def _execute_command(self,cmd):
-        return mockmip.MockMIP._execute_command(self,cmd)
+        return MockMIP._execute_command(self,cmd)
 
 
 pyutilib.services.register_executable(name="optimizer")
