@@ -11,24 +11,24 @@
 import pyutilib.component.core
 pyutilib.component.core.PluginGlobals.push_env( 'coopr.opt' )
 
-from ps import PatternSearch
-from PICO import PICO, MockPICO
-from CBCplugin import CBC, MockCBC
-from GLPK import GLPK, configure_glpk
-import GLPK_old
-from glpk_direct import GLPKDirect
-from CPLEX import CPLEX, MockCPLEX
-from CPLEXDirect import CPLEXDirect
-from GUROBI import GUROBI
-from gurobi_direct import gurobi_direct
-from ASL import ASL, MockASL
-from XPRESS import XPRESS, MockXPRESS
+from coopr.plugins.solvers.ps import PatternSearch
+from coopr.plugins.solvers.PICO import PICO, MockPICO
+from coopr.plugins.solvers.CBCplugin import CBC, MockCBC, configure_cbc
+from coopr.plugins.solvers.GLPK import GLPK, configure_glpk
+import coopr.plugins.solvers.GLPK_old
+from coopr.plugins.solvers.glpk_direct import GLPKDirect
+from coopr.plugins.solvers.CPLEX import CPLEX, MockCPLEX
+from coopr.plugins.solvers.CPLEXDirect import CPLEXDirect
+from coopr.plugins.solvers.GUROBI import GUROBI
+from coopr.plugins.solvers.gurobi_direct import gurobi_direct
+from coopr.plugins.solvers.ASL import ASL, MockASL
+from coopr.plugins.solvers.SCIPAMPL import SCIPAMPL
+from coopr.plugins.solvers.XPRESS import XPRESS, MockXPRESS
 
 #
 # Interrogate the CBC executable to see if it recognizes the -AMPL flag
 #
-import CBCplugin
-CBCplugin.configure()
+configure_cbc()
 
 #
 # Interrogate the glpsol executable to see if it is new enough to allow the new parser logic
