@@ -9,28 +9,28 @@
 #  For more information, see the Coopr README.txt file.
 #  _________________________________________________________________________
 
-
 import logging
-
-from coopr.opt.base import *
-from coopr.opt.results import *
-from coopr.opt.solver import *
-
-from coopr.pyomo.base.numvalue import value
-
-import mockmip
-from pyutilib.misc import Options
-from pyutilib.component.core import alias
-
-logger = logging.getLogger('coopr.plugins')
-
+_glpk_version = None
 try:
     # import all the glp_* functions
-    from glpk.glpkpi import *
+    from glpk import *
     glpk_python_api_exists = True
 except ImportError:
     glpk_python_api_exists = False
 
+from pyutilib.misc import Bunch, Options
+from pyutilib.component.core import alias
+
+from coopr.opt.base import *
+from coopr.opt.base.solvers import _extract_version
+from coopr.opt.results import *
+from coopr.opt.solver import *
+from coopr.pyomo.base.numvalue import value
+from coopr.pyomo.base import Suffix
+
+logger = logging.getLogger('coopr.plugins')
+
+
 
 class GLPKDirect ( OptSolver ):
     """The GLPK LP/MIP solver (direct API plugin)
@@ -91,10 +91,14 @@ class GLPKDirect ( OptSolver ):
         self._capabilities = Options()
         self._capabilities.linear = True
         self._capabilities.integer = True
-        self._capabilities.sos1 = False
-        self._capabilities.sos2 = False
-        self._capabilities.sosn = False
 
+    def version(self):
+        """
+        Returns a tuple describing the solver executable version.
+        """
+        if _glpk_version is None:
+            return _extract_version('')
+        return _glpk_version
 
     def _populate_glpk_instance ( self, model ):
         from coopr.pyomo.base import Var, Objective, Constraint, \
@@ -103,10 +107,11 @@ class GLPKDirect ( OptSolver ):
 
         try:
             lp = glp_create_prob()
-        except Exception, e:
+        except Exception:
+            e = sys.exc_info()[1]
             msg = 'Unable to create GLPK problem instance.  Have you installed' \
             '\n       the Python bindings for GLPK?\n\n\tError message: %s'
-            raise Exception, msg % e
+            raise Exception(msg % e)
 
         objective = sorted( model.active_components( Objective ).values() )[0]
         # so we can correctly map the solution to the correct objective label in _postsolve
@@ -126,7 +131,7 @@ class GLPKDirect ( OptSolver ):
         sos_con_list = model.active_components(SOSConstraint)
         if not(sos1 or sos2 or sosn):
             if len(sos_con_list) > 0:
-                raise Exception, "Solver: glpk_direct does not support SOSConstraint declarations"
+                raise Exception("Solver: glpk_direct does not support SOSConstraint declarations")
         
 
         glp_set_prob_name( lp, model.name )
@@ -182,6 +187,9 @@ class GLPKDirect ( OptSolver ):
                 else:
                     glp_set_col_kind( lp, col, GLP_CV )   # continuous
 
+        model_repn_suffix = model.subcomponent("repn")
+        if (model_repn_suffix is None) or (not model_repn_suffix.type() is Suffix) or (not model_repn_suffix.active is True):
+            raise ValueError("Unable to find an active Suffix with name 'repn' on block: %s" % (model.cname(True)))
         for name in constraint_list.keys():
             constraint_set = constraint_list[ name ]
             if constraint_set.trivial: continue
@@ -193,8 +201,9 @@ class GLPKDirect ( OptSolver ):
                     continue
 
                 offset = 0.0
-                if 0 in constraint.repn:
-                    offset = constraint.repn[0][None]
+                expression = model_repn_suffix.getValue(constraint)
+                if 0 in expression:
+                    offset = expression[0][None]
 
                 lbound = ubound = -offset
 
@@ -219,7 +228,6 @@ class GLPKDirect ( OptSolver ):
                 glp_set_row_name( lp, row, constraint.label )
                 glp_set_row_bnds( lp, row, var_type, lbound, ubound )
 
-                expression = constraint.repn
                 if 1 in expression: # first-order terms
                     keys = sorted( expression[1].keys() )
                     for var_key in keys:
@@ -237,7 +245,7 @@ class GLPKDirect ( OptSolver ):
         glp_load_matrix( lp, coef_count, Ai, Aj, Ar )
 
         for key in objective:
-            expression = objective[ key ].repn
+            expression = model_repn_suffix.getValue(objective[ key ])
             if is_constant( expression ):
                 msg = "Ignoring objective '%s[%s]' which is constant"
                 logger.warning( msg % (str(objective), str(key)) )
@@ -289,11 +297,11 @@ class GLPKDirect ( OptSolver ):
         if len(args) != 1:
             msg = "The glpk_direct plugin method '_presolve' must be supplied "  \
                   "a single problem instance - %s were supplied"
-            raise ValueError, msg % len(args)
+            raise ValueError(msg % len(args))
         elif not isinstance(model, Model):
             msg = "The problem instance supplied to the glpk_direct plugin "     \
                   "'_presolve' method must be of type 'Model'"
-            raise ValueError, msg
+            raise ValueError(msg)
 
         self._populate_glpk_instance( model )
         lp = self._glpk_instance
@@ -357,7 +365,7 @@ class GLPKDirect ( OptSolver ):
             if len(args) != 1:
                 msg = "The glpk_direct _presolve method can only handle a single "\
                       "problem instance - %s were supplied"
-                raise ValueError, msg % len(args)
+                raise ValueError(msg % len(args))
 
             parm.presolve = GLP_OFF
             self.warm_start( model )
@@ -374,7 +382,8 @@ class GLPKDirect ( OptSolver ):
             self.solve_return_code = algorithm( self._glpk_instance, parm )
             end = glp_time()
             self._glpk_solve_time = glp_difftime( end, beg )
-        except Exception, e:
+        except Exception:
+            e = sys.exc_info()[1]
             msg = str(e)
             if 'algorithm' in self.options:
                 msg = "Unexpected error using '%s' algorithm.  Is it the correct "\
@@ -383,6 +392,10 @@ class GLPKDirect ( OptSolver ):
             logger.error( msg )
             raise
 
+        # FIXME: can we get a return code indicating if GLPK had a
+        # significant failure?
+        return Bunch(rc=None, log=None)
+
 
     def _glpk_return_code_to_message ( self ):
         code = self.solve_return_code
@@ -445,7 +458,7 @@ class GLPKDirect ( OptSolver ):
         elif GLP_NOFEAS == status: return SolutionStatus.other
         elif GLP_UNBND  == status: return SolutionStatus.other
         elif GLP_UNDEF  == status: return SolutionStatus.other
-        raise RuntimeError, "Unknown solution status returned by GLPK solver"
+        raise RuntimeError("Unknown solution status returned by GLPK solver")
 
 
     def _glpk_get_solver_status ( self ):
@@ -471,6 +484,12 @@ class GLPKDirect ( OptSolver ):
         bin_variables = glp_get_num_bin( lp )
         int_variables = glp_get_num_int( lp )
 
+        # check suffixes
+        for suffix in self.suffixes:
+            if True:
+                raise RuntimeError("***The glpk_direct solver plugin cannot extract solution suffix="+suffix)
+
+
         tpeak = glp_long()
         glp_mem_usage( None, None, None, tpeak )
         # black magic trickery, thanks to Python's lack of pointers and SWIG's
@@ -539,7 +558,6 @@ class GLPKDirect ( OptSolver ):
                 row = rowvar_map[ row_label ]
                 soln.constraint[ row_label ] = get_row_prim( lp, row )
 
-
         results.solution.insert(soln)
 
         self.results = results
