@@ -22,7 +22,7 @@ from coopr.opt import ResultsFormat, ProblemFormat
 
 coopr.opt.SolverResults.default_print_options.ignore_time = True
 try:
-    cbc = coopr.plugins.mip.CBC(keepFiles=True)
+    cbc = coopr.plugins.mip.CBC(keepfiles=True)
     cbc_available= (not cbc.executable() is None)
 except pyutilib.common.ApplicationError:
     cbc_available=False
@@ -47,9 +47,9 @@ class mock_all(unittest.TestCase):
         os.chdir(currdir)
         pyutilib.services.TempfileManager.tempdir = currdir
         if flag:
-            self.cbc = coopr.plugins.mip.CBC(keepFiles=True)
+            self.cbc = coopr.plugins.mip.CBC(keepfiles=True)
         else:
-            self.cbc = coopr.plugins.mip.MockCBC(keepFiles=True)
+            self.cbc = coopr.plugins.mip.MockCBC(keepfiles=True)
         self.cbc.suffixes=['.*']
 
     def tearDown(self):
