@@ -28,7 +28,7 @@ except pyutilib.common.ApplicationError:
     pico_convert_available=False
 
 try:
-    pico = coopr.plugins.mip.PICO(keepFiles=True)
+    pico = coopr.plugins.mip.PICO(keepfiles=True)
     pico_available= (not pico.executable() is None)
 except pyutilib.common.ApplicationError:
     pico_available=False
@@ -46,9 +46,9 @@ class mock_all(unittest.TestCase):
         os.chdir(currdir)
         pyutilib.services.TempfileManager.tempdir = currdir
         if flag:
-            self.pico = coopr.plugins.mip.PICO(keepFiles=True)
+            self.pico = coopr.plugins.mip.PICO(keepfiles=True)
         else:
-            self.pico = coopr.plugins.mip.MockPICO(keepFiles=True)
+            self.pico = coopr.plugins.mip.MockPICO(keepfiles=True)
         self.pico.suffixes=['.*']
 
     def tearDown(self):
